@@ -1,13 +1,34 @@
 open Import
 
+module Selected_context = struct
+  let arg =
+    let ctx_name_conv =
+      let parse ctx_name =
+        match Context_name.of_string_opt ctx_name with
+        | None -> Error (`Msg (Printf.sprintf "Invalid context name %S" ctx_name))
+        | Some ctx_name -> Ok ctx_name
+      in
+      let print ppf t = Stdlib.Format.fprintf ppf "%s" (Context_name.to_string t) in
+      Arg.conv ~docv:"context" (parse, print)
+    in
+    Arg.(
+      value
+      & opt ctx_name_conv Context_name.default
+      & info
+          [ "context" ]
+          ~docv:"CONTEXT"
+          ~doc:"Select the Dune build context that will be used to return information")
+  ;;
+end
+
 module Server : sig
-  val dump : string -> unit Fiber.t
-  val dump_dot_merlin : string -> unit Fiber.t
+  val dump : selected_context:Context_name.t -> string -> unit Fiber.t
+  val dump_dot_merlin : selected_context:Context_name.t -> string -> unit Fiber.t
 
   (** Once started the server will wait for commands on stdin, read the
       requested merlin dot file and return its content on stdout. The server
       will halt when receiving EOF of a bad csexp. *)
-  val start : unit -> unit Fiber.t
+  val start : selected_context:Context_name.t -> unit -> unit Fiber.t
 end = struct
   open Fiber.O
 
@@ -129,50 +150,53 @@ end = struct
       |> error
   ;;
 
-  let to_local file =
+  let to_local ~selected_context file =
     match to_local file with
     | Error s -> Fiber.return (Error s)
     | Ok file ->
-      let+ workspace = Memo.run (Workspace.workspace ()) in
-      let module Context_name = Dune_engine.Context_name in
-      (match workspace.merlin_context with
-       | None -> Error "no merlin context configured"
-       | Some context ->
-         Ok (Path.Build.append_local (Context_name.build_dir context) file))
+      (match Dune_engine.Context_name.is_default selected_context with
+       | false ->
+         Fiber.return
+           (Ok (Path.Build.append_local (Context_name.build_dir selected_context) file))
+       | true ->
+         let+ workspace = Memo.run (Workspace.workspace ()) in
+         (match workspace.merlin_context with
+          | None -> Error "no merlin context configured"
+          | Some context ->
+            Ok (Path.Build.append_local (Context_name.build_dir context) file)))
   ;;
 
-  let print_merlin_conf file =
-    let+ answer =
-      let+ file = to_local file in
-      match file with
-      | Error s -> Merlin_conf.make_error s
-      | Ok file -> load_merlin_file file
-    in
-    Merlin_conf.to_stdout answer
+  let print_merlin_conf ~selected_context file =
+    to_local ~selected_context file
+    >>| (function
+     | Error s -> Merlin_conf.make_error s
+     | Ok file -> load_merlin_file file)
+    >>| Merlin_conf.to_stdout
   ;;
 
-  let dump s =
-    let+ file = to_local s in
-    match file with
+  let dump ~selected_context s =
+    to_local ~selected_context s
+    >>| function
     | Error mess -> Printf.eprintf "%s\n%!" mess
     | Ok path -> get_merlin_files_paths path |> List.iter ~f:Merlin.Processed.print_file
   ;;
 
-  let dump_dot_merlin s =
-    let+ file = to_local s in
-    match file with
+  let dump_dot_merlin ~selected_context s =
+    to_local ~selected_context s
+    >>| function
     | Error mess -> Printf.eprintf "%s\n%!" mess
     | Ok path ->
       let files = get_merlin_files_paths path in
       Merlin.Processed.print_generic_dot_merlin files
   ;;
 
-  let start () =
+  let start ~selected_context () =
+    let open Fiber.O in
     let rec main () =
       match Commands.read_input stdin with
       | Halt -> Fiber.return ()
       | File path ->
-        let* () = print_merlin_conf path in
+        let* () = print_merlin_conf ~selected_context path in
         main ()
       | Unknown msg ->
         Merlin_conf.to_stdout (Merlin_conf.make_error msg);
@@ -194,11 +218,16 @@ module Dump_config = struct
 
   let term =
     let+ builder = Common.Builder.term
-    and+ dir = Arg.(value & pos 0 dir "" & info [] ~docv:"PATH") in
-    let builder = Common.Builder.forbid_builds builder in
-    let builder = Common.Builder.disable_log_file builder in
-    let common, config = Common.init builder in
-    Scheduler.go ~common ~config (fun () -> Server.dump dir)
+    and+ dir = Arg.(value & pos 0 dir "" & info [] ~docv:"PATH")
+    and+ selected_context = Selected_context.arg in
+    let common, config =
+      let builder =
+        let builder = Common.Builder.forbid_builds builder in
+        Common.Builder.disable_log_file builder
+      in
+      Common.init builder
+    in
+    Scheduler.go ~common ~config (fun () -> Server.dump ~selected_context dir)
   ;;
 
   let command = Cmd.v info term
@@ -220,11 +249,16 @@ let man =
 let start_session_info name = Cmd.info name ~doc ~man
 
 let start_session_term =
-  let+ builder = Common.Builder.term in
-  let builder = Common.Builder.forbid_builds builder in
-  let builder = Common.Builder.disable_log_file builder in
-  let common, config = Common.init builder in
-  Scheduler.go ~common ~config Server.start
+  let+ builder = Common.Builder.term
+  and+ selected_context = Selected_context.arg in
+  let common, config =
+    let builder =
+      let builder = Common.Builder.forbid_builds builder in
+      Common.Builder.disable_log_file builder
+    in
+    Common.init builder
+  in
+  Scheduler.go ~common ~config (Server.start ~selected_context)
 ;;
 
 let command = Cmd.v (start_session_info "ocaml-merlin") start_session_term
@@ -258,14 +292,18 @@ module Dump_dot_merlin = struct
             ~doc:
               "The path to the folder of which the configuration should be printed. \
                Defaults to the current directory.")
+    and+ selected_context = Selected_context.arg in
+    let common, config =
+      let builder =
+        let builder = Common.Builder.forbid_builds builder in
+        Common.Builder.disable_log_file builder
+      in
+      Common.init builder
     in
-    let builder = Common.Builder.forbid_builds builder in
-    let builder = Common.Builder.disable_log_file builder in
-    let common, config = Common.init builder in
     Scheduler.go ~common ~config (fun () ->
       match path with
-      | Some s -> Server.dump_dot_merlin s
-      | None -> Server.dump_dot_merlin ".")
+      | Some s -> Server.dump_dot_merlin ~selected_context s
+      | None -> Server.dump_dot_merlin ~selected_context ".")
   ;;
 
   let command = Cmd.v info term
