@@ -19,8 +19,12 @@ let info = Cmd.info "top" ~doc ~man
 
 let link_deps sctx link =
   let open Memo.O in
+  let* lib_config =
+    let+ ocaml = Super_context.context sctx |> Context.ocaml in
+    ocaml.lib_config
+  in
   Memo.parallel_map link ~f:(fun t ->
-    Dune_rules.Lib_flags.link_deps sctx t Dune_rules.Link_mode.Byte)
+    Dune_rules.Lib_flags.link_deps sctx t Dune_rules.Link_mode.Byte lib_config)
   >>| List.concat
 ;;
 
@@ -41,15 +45,16 @@ let term =
   Scheduler.go ~common ~config (fun () ->
     let open Fiber.O in
     let* setup = Import.Main.setup () in
-    Build_system.run_exn (fun () ->
+    build_exn (fun () ->
       let open Memo.O in
       let* setup = setup in
       let sctx =
         Dune_engine.Context_name.Map.find setup.scontexts ctx_name |> Option.value_exn
       in
+      let context = Super_context.context sctx in
       let* libs =
         let dir =
-          let build_dir = Super_context.context sctx |> Context.build_dir in
+          let build_dir = Context.build_dir context in
           Path.Build.relative build_dir (Common.prefix_target common dir)
         in
         let* db =
@@ -62,7 +67,13 @@ let term =
       let* requires =
         Dune_rules.Resolve.Memo.read_memo (Dune_rules.Lib.closure ~linking:true libs)
       in
-      let include_paths = Dune_rules.Lib_flags.L.toplevel_include_paths requires in
+      let* lib_config =
+        let+ ocaml = Context.ocaml context in
+        ocaml.lib_config
+      in
+      let include_paths =
+        Dune_rules.Lib_flags.L.toplevel_include_paths requires lib_config
+      in
       let+ files_to_load = files_to_load_of_requires sctx requires in
       Dune_rules.Toplevel.print_toplevel_init_file
         { include_paths; files_to_load; uses = []; pp = None; ppx = None; code = [] }))
@@ -116,7 +127,10 @@ module Module = struct
       in
       let private_obj_dir = Top_module.private_obj_dir ctx mod_ in
       let include_paths =
-        let libs = Dune_rules.Lib_flags.L.toplevel_include_paths requires in
+        let libs =
+          let lib_config = (Compilation_context.ocaml cctx).lib_config in
+          Dune_rules.Lib_flags.L.toplevel_include_paths requires lib_config
+        in
         Path.Set.add libs (Path.build (Obj_dir.byte_dir private_obj_dir))
       in
       let files_to_load () =
@@ -124,44 +138,46 @@ module Module = struct
           Memo.fork_and_join
             (fun () -> files_to_load_of_requires sctx requires)
             (fun () ->
-              let cmis () =
-                let glob =
-                  Dune_engine.File_selector.of_glob
-                    ~dir:(Path.build (Obj_dir.byte_dir private_obj_dir))
-                    (Dune_lang.Glob.of_string_exn Loc.none "*.cmi")
-                in
-                let* files = Build_system.eval_pred glob in
-                Memo.parallel_iter (Filename_set.to_list files) ~f:Build_system.build_file
-              in
-              let cmos () =
-                let obj_dir = Compilation_context.obj_dir cctx in
-                let dep_graph = (Compilation_context.dep_graphs cctx).impl in
-                let* modules =
-                  let graph =
-                    Dune_rules.Dep_graph.top_closed_implementations dep_graph [ module_ ]
-                  in
-                  let+ modules, _ = Action_builder.evaluate_and_collect_facts graph in
-                  modules
-                in
-                let cmos =
-                  let module Module = Dune_rules.Module in
-                  let module Module_name = Dune_rules.Module_name in
-                  let module_obj_name = Module.obj_name module_ in
-                  List.filter_map modules ~f:(fun m ->
-                    let obj_dir =
-                      if Module_name.Unique.equal module_obj_name (Module.obj_name m)
-                      then private_obj_dir
-                      else obj_dir
-                    in
-                    Obj_dir.Module.cm_file obj_dir m ~kind:(Ocaml Cmo)
-                    |> Option.map ~f:Path.build)
-                in
-                let+ (_ : Dep.Facts.t) =
-                  Build_system.build_deps (Dep.Set.of_files cmos)
-                in
-                cmos
-              in
-              Memo.fork_and_join_unit cmis cmos)
+               let cmis () =
+                 let glob =
+                   Dune_engine.File_selector.of_glob
+                     ~dir:(Path.build (Obj_dir.byte_dir private_obj_dir))
+                     (Dune_lang.Glob.of_string_exn Loc.none "*.cmi")
+                 in
+                 let* files = Build_system.eval_pred glob in
+                 Memo.parallel_iter
+                   (Filename_set.to_list files)
+                   ~f:Build_system.build_file
+               in
+               let cmos () =
+                 let obj_dir = Compilation_context.obj_dir cctx in
+                 let dep_graph = (Compilation_context.dep_graphs cctx).impl in
+                 let* modules =
+                   let graph =
+                     Dune_rules.Dep_graph.top_closed_implementations dep_graph [ module_ ]
+                   in
+                   let+ modules, _ = Action_builder.evaluate_and_collect_facts graph in
+                   modules
+                 in
+                 let cmos =
+                   let module Module = Dune_rules.Module in
+                   let module Module_name = Dune_rules.Module_name in
+                   let module_obj_name = Module.obj_name module_ in
+                   List.filter_map modules ~f:(fun m ->
+                     let obj_dir =
+                       if Module_name.Unique.equal module_obj_name (Module.obj_name m)
+                       then private_obj_dir
+                       else obj_dir
+                     in
+                     Obj_dir.Module.cm_file obj_dir m ~kind:(Ocaml Cmo)
+                     |> Option.map ~f:Path.build)
+                 in
+                 let+ (_ : Dep.Facts.t) =
+                   Build_system.build_deps (Dep.Set.of_files cmos)
+                 in
+                 cmos
+               in
+               Memo.fork_and_join_unit cmis cmos)
         in
         libs @ modules
       in
@@ -200,7 +216,7 @@ module Module = struct
     Scheduler.go ~common ~config (fun () ->
       let open Fiber.O in
       let* setup = Import.Main.setup () in
-      Build_system.run_exn (fun () ->
+      build_exn (fun () ->
         let open Memo.O in
         let* setup = setup in
         let sctx =
