@@ -42,407 +42,465 @@ let local_part p = Path.local_part p |> Path.Local.to_dyn |> print_dyn
 let%expect_test _ =
   let p = Path.(relative root) "foo" in
   descendant p ~of_:p;
-  [%expect {|
-  Some In_source_tree "."
+  [%expect
+    {|
+  Some (In_source_tree ".")
   |}]
 ;;
 
 let%expect_test _ =
   (* different strings but same length *)
   descendant (r "foo") ~of_:(r "bar");
-  [%expect {|
+  [%expect
+    {|
   None
   |}]
 ;;
 
 let%expect_test _ =
   is_descendant (r "foo") ~of_:(r "foo");
-  [%expect {|
+  [%expect
+    {|
   true
   |}]
 ;;
 
 let%expect_test _ =
   is_descendant (r "foo") ~of_:(r "foo/");
-  [%expect {|
+  [%expect
+    {|
   true
   |}]
 ;;
 
 let%expect_test _ =
   is_descendant (r "foo/") ~of_:(r "foo");
-  [%expect {|
+  [%expect
+    {|
   true
   |}]
 ;;
 
 let%expect_test _ =
   is_descendant (r "foo") ~of_:(r "bar");
-  [%expect {|
+  [%expect
+    {|
   false
   |}]
 ;;
 
 let%expect_test _ =
   is_descendant (r "foo") ~of_:(r "bar/");
-  [%expect {|
+  [%expect
+    {|
 false
 |}]
 ;;
 
 let%expect_test _ =
   is_descendant (r "foo/") ~of_:(r "bar");
-  [%expect {|
+  [%expect
+    {|
 false
 |}]
 ;;
 
 let%expect_test _ =
   is_descendant (r "glob/foo") ~of_:(r "glob");
-  [%expect {|
+  [%expect
+    {|
 true
 |}]
 ;;
 
 let%expect_test _ =
   is_descendant (r "glob/foo") ~of_:(r "glob/");
-  [%expect {|
+  [%expect
+    {|
 true
 |}]
 ;;
 
 let%expect_test _ =
   is_descendant (e "/foo/bar") ~of_:(e "/foo");
-  [%expect {|
+  [%expect
+    {|
 false
 |}]
 ;;
 
 let%expect_test _ =
   is_descendant (e "/foo/bar") ~of_:(e "/foo/bar");
-  [%expect {|
+  [%expect
+    {|
 false
 |}]
 ;;
 
 let%expect_test _ =
   is_descendant (e "/foo/bar") ~of_:(e "/foo/bar/");
-  [%expect {|
+  [%expect
+    {|
 false
 |}]
 ;;
 
 let%expect_test _ =
   is_descendant (e "/foo/bar/") ~of_:(e "/foo/bar");
-  [%expect {|
+  [%expect
+    {|
 false
 |}]
 ;;
 
 let%expect_test _ =
   is_descendant (e "/foo/bar") ~of_:(e "/");
-  [%expect {|
+  [%expect
+    {|
 false
 |}]
 ;;
 
 let%expect_test _ =
   descendant (r "foo") ~of_:(r "foo/");
-  [%expect {|
-Some In_source_tree "."
+  [%expect
+    {|
+Some (In_source_tree ".")
 |}]
 ;;
 
 let%expect_test _ =
   descendant (r "foo/") ~of_:(r "foo");
-  [%expect {|
-Some In_source_tree "."
+  [%expect
+    {|
+Some (In_source_tree ".")
 |}]
 ;;
 
 let%expect_test _ =
   descendant (r "foo/bar") ~of_:(r "foo");
-  [%expect {|
-Some In_source_tree "bar"
+  [%expect
+    {|
+Some (In_source_tree "bar")
 |}]
 ;;
 
 let%expect_test _ =
   descendant Path.root ~of_:(r "foo");
-  [%expect {|
+  [%expect
+    {|
 None
 |}]
 ;;
 
 let%expect_test _ =
   descendant Path.root ~of_:Path.root;
-  [%expect {|
-Some In_source_tree "."
+  [%expect
+    {|
+Some (In_source_tree ".")
 |}]
 ;;
 
 let%expect_test _ =
   descendant (r "foo") ~of_:Path.root;
-  [%expect {|
-Some In_source_tree "foo"
+  [%expect
+    {|
+Some (In_source_tree "foo")
 |}]
 ;;
 
 let%expect_test _ =
   descendant (Path.relative build_dir "foo") ~of_:root;
-  [%expect {|
+  [%expect
+    {|
 None
 |}]
 ;;
 
 let%expect_test _ =
   descendant (Path.relative build_dir "foo") ~of_:(Path.of_string "/foo/bar");
-  [%expect {|
+  [%expect
+    {|
 None
 |}]
 ;;
 
 let%expect_test _ =
   descendant (Path.relative build_dir "foo/bar") ~of_:build_dir;
-  [%expect {|
-Some In_source_tree "foo/bar"
+  [%expect
+    {|
+Some (In_source_tree "foo/bar")
 |}]
 ;;
 
 let%expect_test _ =
   descendant (Path.relative build_dir "foo/bar") ~of_:(Path.relative build_dir "foo");
-  [%expect {|
-Some In_source_tree "bar"
+  [%expect
+    {|
+Some (In_source_tree "bar")
 |}]
 ;;
 
 let%expect_test _ =
   descendant (Path.relative build_dir "foo/bar") ~of_:(Path.relative build_dir "foo");
-  [%expect {|
-Some In_source_tree "bar"
+  [%expect
+    {|
+Some (In_source_tree "bar")
 |}]
 ;;
 
 let%expect_test _ =
   descendant (Path.of_string "/foo/bar") ~of_:(Path.of_string "/foo");
-  [%expect {|
+  [%expect
+    {|
 None
 |}]
 ;;
 
 let%expect_test _ =
   explode "a/b/c";
-  [%expect {|
+  [%expect
+    {|
 Some [ "a"; "b"; "c" ]
 |}]
 ;;
 
 let%expect_test _ =
   explode "a/b";
-  [%expect {|
+  [%expect
+    {|
 Some [ "a"; "b" ]
 |}]
 ;;
 
 let%expect_test _ =
   explode "a";
-  [%expect {|
+  [%expect
+    {|
 Some [ "a" ]
 |}]
 ;;
 
 let%expect_test _ =
   explode "";
-  [%expect {|
+  [%expect
+    {|
 Some []
 |}]
 ;;
 
 let%expect_test _ =
   reach "/foo/baz" ~from:"/foo/bar";
-  [%expect {|
+  [%expect
+    {|
 "/foo/baz"
 |}]
 ;;
 
 let%expect_test _ =
   reach "/foo/bar" ~from:"baz";
-  [%expect {|
+  [%expect
+    {|
 "/foo/bar"
 |}]
 ;;
 
 let%expect_test _ =
   reach "bar/foo" ~from:"bar/baz/y";
-  [%expect {|
+  [%expect
+    {|
 "../../foo"
 |}]
 ;;
 
 let%expect_test _ =
   reach "foo" ~from:"foo";
-  [%expect {|
+  [%expect
+    {|
 "."
 |}]
 ;;
 
 let%expect_test _ =
   reach "bar/foo" ~from:"bar/foo";
-  [%expect {|
+  [%expect
+    {|
 "."
 |}]
 ;;
 
 let%expect_test _ =
   reach "a/b/x" ~from:"a/b/y";
-  [%expect {|
+  [%expect
+    {|
 "../x"
 |}]
 ;;
 
 let%expect_test _ =
   reach "a/b" ~from:"a/b/x";
-  [%expect {|
+  [%expect
+    {|
 ".."
 |}]
 ;;
 
 let%expect_test _ =
   reach "a/b/x" ~from:"a/b";
-  [%expect {|
+  [%expect
+    {|
 "x"
 |}]
 ;;
 
 let%expect_test _ =
   reach "a/b/x/z" ~from:"a/b/y";
-  [%expect {|
+  [%expect
+    {|
 "../x/z"
 |}]
 ;;
 
 let%expect_test _ =
   reach "a/b/y" ~from:"a/b/x/z";
-  [%expect {|
+  [%expect
+    {|
 "../../y"
 |}]
 ;;
 
 let%expect_test _ =
   reach "a/bbb" ~from:"a/b";
-  [%expect {|
+  [%expect
+    {|
 "../bbb"
 |}]
 ;;
 
 let%expect_test _ =
   reach "" ~from:"";
-  [%expect {|
+  [%expect
+    {|
 "."
 |}]
 ;;
 
 let%expect_test _ =
   reach "" ~from:"foo";
-  [%expect {|
+  [%expect
+    {|
 ".."
 |}]
 ;;
 
 let%expect_test _ =
   reach "foo" ~from:"";
-  [%expect {|
+  [%expect
+    {|
 "foo"
 |}]
 ;;
 
 let%expect_test _ =
   reach "x/foo" ~from:"bar/x";
-  [%expect {|
+  [%expect
+    {|
 "../../x/foo"
 |}]
 ;;
 
 let%expect_test _ =
   reach "a/x" ~from:"x/b";
-  [%expect {|
+  [%expect
+    {|
 "../../a/x"
 |}]
 ;;
 
 let%expect_test _ =
   reach "default/META.foo" ~from:"default";
-  [%expect {|
+  [%expect
+    {|
 "META.foo"
 |}]
 ;;
 
 let%expect_test _ =
   reach "default/av" ~from:"default/avdevice";
-  [%expect {|
+  [%expect
+    {|
 "../av"
 |}]
 ;;
 
 let%expect_test _ =
   relative (Path.of_string "relative") "/absolute/path";
-  [%expect {|
+  [%expect
+    {|
 External "/absolute/path"
 |}]
 ;;
 
 let%expect_test _ =
   relative (Path.of_string "/abs1") "/abs2";
-  [%expect {|
+  [%expect
+    {|
 External "/abs2"
 |}]
 ;;
 
 let%expect_test _ =
   relative (of_string "/abs1") "";
-  [%expect {|
+  [%expect
+    {|
 External "/abs1"
 |}]
 ;;
 
 let%expect_test _ =
   relative root "/absolute/path";
-  [%expect {|
+  [%expect
+    {|
 External "/absolute/path"
 |}]
 ;;
 
 let%expect_test _ =
   of_filename_relative_to_initial_cwd "/absolute/path";
-  [%expect {|
+  [%expect
+    {|
 External "/absolute/path"
 |}]
 ;;
 
 let%expect_test _ =
   Path.is_managed (e "relative/path") |> Dyn.bool |> print_dyn;
-  [%expect {|
+  [%expect
+    {|
 false
 |}]
 ;;
 
 let%expect_test _ =
   append_source Path.build_dir (Path.Source.relative Path.Source.root "foo");
-  [%expect {|
+  [%expect
+    {|
 In_build_dir "foo"
 |}]
 ;;
 
 let%expect_test _ =
   append_source Path.root (Path.Source.relative Path.Source.root "foo");
-  [%expect {|
+  [%expect
+    {|
 In_source_tree "foo"
 |}]
 ;;
 
 let%expect_test _ =
   append_source (Path.of_string "/root") (Path.Source.relative Path.Source.root "foo");
-  [%expect {|
+  [%expect
+    {|
 External "/root/foo"
 |}]
 ;;
@@ -453,54 +511,61 @@ let%expect_test _ =
 [@@expect.uncaught_exn
   {|
   ( "(\"Path.rm_rf called on external dir\",\
-   \n{ t = External \"/does/not/exist/foo/bar/baz\" })") |}]
+   \n { t = External \"/does/not/exist/foo/bar/baz\" })") |}]
 ;;
 
 let%expect_test _ =
   drop_build_context (Path.relative Path.build_dir "foo/bar");
-  [%expect {|
-Some In_source_tree "bar"
+  [%expect
+    {|
+Some (In_source_tree "bar")
 |}]
 ;;
 
 let%expect_test _ =
   drop_build_context (Path.of_string "foo/bar");
-  [%expect {|
+  [%expect
+    {|
 None
 |}]
 ;;
 
 let%expect_test _ =
   drop_build_context (e "/foo/bar");
-  [%expect {|
+  [%expect
+    {|
 None
 |}]
 ;;
 
 let%expect_test _ =
   drop_build_context Path.build_dir;
-  [%expect {|
+  [%expect
+    {|
 None
 |}]
 ;;
 
 let%expect_test _ =
   Path.is_in_build_dir Path.build_dir |> Dyn.bool |> print_dyn;
-  [%expect {|
+  [%expect
+    {|
 true
 |}]
 ;;
 
 let%expect_test _ =
   Path.is_strict_descendant_of_build_dir Path.build_dir |> Dyn.bool |> print_dyn;
-  [%expect {|
+  [%expect
+    {|
 false
 |}]
 ;;
 
 let%expect_test _ =
   Path.reach_for_running Path.build_dir ~from:Path.root |> Dyn.string |> print_dyn;
-  [%expect {|
+  [%expect
+    {|
 "./_build"
 |}]
 ;;
@@ -509,28 +574,32 @@ let%expect_test _ =
   reach_for_running
     (Path.relative build_dir "foo/baz")
     ~from:(Path.relative build_dir "foo/bar/baz");
-  [%expect {|
+  [%expect
+    {|
 "../../baz"
 |}]
 ;;
 
 let%expect_test _ =
   reach_for_running (e "/fake/path") ~from:(Path.relative build_dir "foo/bar/baz");
-  [%expect {|
+  [%expect
+    {|
 "/fake/path"
 |}]
 ;;
 
 let%expect_test _ =
   reach_for_running (Path.relative root "foo") ~from:(Path.relative root "foo");
-  [%expect {|
+  [%expect
+    {|
 "./."
 |}]
 ;;
 
 let%expect_test _ =
   relative Path.root "_build";
-  [%expect {|
+  [%expect
+    {|
 In_build_dir "."
 |}]
 ;;
@@ -538,21 +607,24 @@ In_build_dir "."
 let%expect_test _ =
   (* This is not right, but kind of annoying to fix :/ *)
   relative (r "foo") "../_build";
-  [%expect {|
+  [%expect
+    {|
 In_build_dir "."
 |}]
 ;;
 
 let%expect_test _ =
   local_part (Path.of_string "/c/d");
-  [%expect {|
+  [%expect
+    {|
 "c/d"
 |}]
 ;;
 
 let%expect_test _ =
   local_part (r "c/d");
-  [%expect {|
+  [%expect
+    {|
 "c/d"
 |}]
 ;;
@@ -561,7 +633,8 @@ let%expect_test _ =
   Path.Build.extract_first_component Path.Build.root
   |> Dyn.(option (pair string Local.to_dyn))
   |> print_dyn;
-  [%expect {|
+  [%expect
+    {|
 None
 |}]
 ;;
