@@ -10,8 +10,8 @@ module Progress = struct
     }
 
   let equal
-    { number_of_rules_discovered; number_of_rules_executed; number_of_rules_failed }
-    t
+        { number_of_rules_discovered; number_of_rules_executed; number_of_rules_failed }
+        t
     =
     Int.equal number_of_rules_discovered t.number_of_rules_discovered
     && Int.equal number_of_rules_executed t.number_of_rules_executed
@@ -206,6 +206,7 @@ end = struct
       (* Fact: alias [a] expands to the set of file-digest pairs [digests] *)
       Dep.Fact.alias a digests
     | File f ->
+      (* Not necessarily a file, can also be a directory. *)
       let+ digest = build_file f in
       (* Fact: file [f] has digest [digest] *)
       Dep.Fact.file f digest
@@ -245,14 +246,14 @@ end = struct
 
   (* The current version of the rule digest scheme. We should increment it when
      making any changes to the scheme, to avoid collisions. *)
-  let rule_digest_version = 21
+  let rule_digest_version = 22
 
   let compute_rule_digest
-    (rule : Rule.t)
-    ~facts
-    ~action
-    ~sandbox_mode
-    ~execution_parameters
+        (rule : Rule.t)
+        ~facts
+        ~action
+        ~sandbox_mode
+        ~execution_parameters
     =
     let { Action.Full.action
         ; env
@@ -325,14 +326,14 @@ end = struct
   ;;
 
   let execute_action_for_rule
-    ~rule_kind
-    ~rule_digest
-    ~action
-    ~facts
-    ~loc
-    ~execution_parameters
-    ~sandbox_mode
-    ~(targets : Targets.Validated.t)
+        ~rule_kind
+        ~rule_digest
+        ~action
+        ~facts
+        ~loc
+        ~execution_parameters
+        ~sandbox_mode
+        ~(targets : Targets.Validated.t)
     : Exec_result.t Fiber.t
     =
     let open Fiber.O in
@@ -408,44 +409,41 @@ end = struct
           Pending_targets.remove targets;
           Fiber.return ())
       (fun () ->
-        with_locks locks ~f:(fun () ->
-          let* action_exec_result =
-            let input =
-              { Action_exec.root
-              ; context (* can be derived from the root *)
-              ; env
-              ; targets = Some targets
-              ; rule_loc = loc
-              ; execution_parameters
-              ; action
-              }
-            in
-            match (Build_config.get ()).action_runner input with
-            | None ->
-              let build_deps deps = Memo.run (build_deps deps) in
-              Action_exec.exec input ~build_deps
-            | Some runner -> Action_runner.exec_action runner input
-          in
-          let* action_exec_result = Action_exec.Exec_result.ok_exn action_exec_result in
-          let* () =
-            match sandbox with
-            | None -> Fiber.return ()
-            | Some sandbox ->
-              (* The stamp file for anonymous actions is always created outside
+         with_locks locks ~f:(fun () ->
+           let* action_exec_result =
+             let input =
+               { Action_exec.root
+               ; context (* can be derived from the root *)
+               ; env
+               ; targets = Some targets
+               ; rule_loc = loc
+               ; execution_parameters
+               ; action
+               }
+             in
+             let build_deps deps = Memo.run (build_deps deps) in
+             Action_exec.exec input ~build_deps
+           in
+           let* action_exec_result = Action_exec.Exec_result.ok_exn action_exec_result in
+           let* () =
+             match sandbox with
+             | None -> Fiber.return ()
+             | Some sandbox ->
+               (* The stamp file for anonymous actions is always created outside
                  the sandbox, so we can't move it. *)
-              let should_be_skipped =
-                match rule_kind with
-                | Normal_rule -> fun (_ : Path.Build.t) -> false
-                | Anonymous_action { stamp_file; _ } -> Path.Build.equal stamp_file
-              in
-              Sandbox.move_targets_to_build_dir sandbox ~should_be_skipped ~targets
-          in
-          let+ produced_targets =
-            maybe_async_rule_file_op (fun () -> Targets.Produced.of_validated targets)
-          in
-          match produced_targets with
-          | Ok produced_targets -> { Exec_result.produced_targets; action_exec_result }
-          | Error error -> User_error.raise ~loc (Targets.Produced.Error.message error)))
+               let should_be_skipped =
+                 match rule_kind with
+                 | Normal_rule -> fun (_ : Path.Build.t) -> false
+                 | Anonymous_action { stamp_file; _ } -> Path.Build.equal stamp_file
+               in
+               Sandbox.move_targets_to_build_dir sandbox ~should_be_skipped ~targets
+           in
+           let+ produced_targets =
+             maybe_async_rule_file_op (fun () -> Targets.Produced.of_validated targets)
+           in
+           match produced_targets with
+           | Ok produced_targets -> { Exec_result.produced_targets; action_exec_result }
+           | Error error -> User_error.raise ~loc (Targets.Produced.Error.message error)))
   ;;
 
   let promote_targets ~rule_mode ~targets ~promote_source =
@@ -678,7 +676,7 @@ end = struct
 
   (* Returns the action's stdout or the empty string if [capture_stdout = false]. *)
   let execute_action_generic_stage2_impl
-    { Anonymous_action.action = act; deps; capture_stdout; digest }
+        { Anonymous_action.action = act; deps; capture_stdout; digest }
     =
     let target =
       let dir =
@@ -725,12 +723,12 @@ end = struct
 
   (* The current version of the action digest scheme. We should increment it when
      making any changes to the scheme, to avoid collisions. *)
-  let action_digest_version = 1
+  let action_digest_version = 2
 
   let execute_action_generic
-    ~observing_facts
-    (act : Rule.Anonymous_action.t)
-    ~capture_stdout
+        ~observing_facts
+        (act : Rule.Anonymous_action.t)
+        ~capture_stdout
     =
     (* We memoize the execution of anonymous actions, both via the persistent
        mechanism for not re-running build rules between invocations of [dune
@@ -858,10 +856,14 @@ end = struct
 
             rleshchinskiy: Is this digest ever used? [build_dir] discards it and do we
             (or should we) ever use [build_file] to build directories? Perhaps this could
-            be split in two memo tables, one for files and one for directories. *)
+            be split in two memo tables, one for files and one for directories.
+
+            ElectreAAS: a lot of functions are called [build_file] or [create_file]
+            even though they also handle directories, this is expected.
+            Also yes this digest is used by [Exported.build_dep] defined above. *)
          (match Cached_digest.build_file ~allow_dirs:true path with
           | Ok digest -> digest, Dir_target { targets }
-          (* Must be a directory target *)
+          (* Must be a directory target. *)
           | Error _ ->
             (* CR-someday amokhov: The most important reason we end up here is
                [No_such_file]. I think some of the outcomes above are impossible
@@ -923,8 +925,9 @@ end = struct
       >>= Memo.parallel_map ~f:(fun (loc, definition) ->
         Memo.push_stack_frame
           (fun () ->
-            Action_builder.evaluate_and_collect_facts (dep_on_alias_definition definition)
-            >>| snd)
+             Action_builder.evaluate_and_collect_facts
+               (dep_on_alias_definition definition)
+             >>| snd)
           ~human_readable_description:(fun () -> Alias.describe alias ~loc))
     in
     Dep.Facts.group_paths_as_fact_files l
@@ -1063,7 +1066,8 @@ let file_exists fn =
       (Path.Build.Map.mem rules_here.by_file_targets (Path.as_in_build_dir_exn fn))
   | Build_under_directory_target { directory_target_ancestor } ->
     let+ path_map = build_dir (Path.build directory_target_ancestor) in
-    Targets.Produced.mem path_map (Path.as_in_build_dir_exn fn)
+    (* Note that in the case of directory targets, we also check if directories exist. *)
+    Targets.Produced.mem_any path_map (Path.as_in_build_dir_exn fn)
 ;;
 
 let files_of ~dir =
@@ -1106,12 +1110,7 @@ let report_early_exn exn =
     let+ () = State.add_errors errors
     and+ () =
       match !Clflags.stop_on_first_error with
-      | true ->
-        let* () =
-          (Build_config.get ()).action_runners ()
-          |> Fiber.parallel_iter ~f:Action_runner.cancel_build
-        in
-        Scheduler.cancel_current_build ()
+      | true -> Scheduler.cancel_current_build ()
       | false -> Fiber.return ()
     in
     (match !Clflags.report_errors_config with
@@ -1131,7 +1130,6 @@ let handle_final_exns exns =
 
 let run f =
   let open Fiber.O in
-  Hooks.End_of_build.once Diff_promotion.finalize;
   let* () = State.reset_progress () in
   let* () = State.reset_errors () in
   let f () =
@@ -1170,6 +1168,11 @@ let build_file p =
   ()
 ;;
 
+let build_dir p =
+  let+ (_ : Digest.t Targets.Produced.t) = build_dir p in
+  ()
+;;
+
 let with_file p ~f =
   let+ () = build_file p in
   f p
