@@ -6,10 +6,6 @@ module Timestamp = Event.Timestamp
 module Action_output_on_success = Execution_parameters.Action_output_on_success
 module Action_output_limit = Execution_parameters.Action_output_limit
 
-let with_directory_annot =
-  User_message.Annots.Key.create ~name:"with-directory" Path.to_dyn
-;;
-
 let limit_output = Dune_output_truncation.limit_output ~message:"TRUNCATED BY DUNE"
 
 module Failure_mode = struct
@@ -203,12 +199,12 @@ let default_metadata =
 ;;
 
 let create_metadata
-  ?loc
-  ?(annots = default_metadata.annots)
-  ?name
-  ?(categories = default_metadata.categories)
-  ?(purpose = Internal_job)
-  ()
+      ?loc
+      ?(annots = default_metadata.annots)
+      ?name
+      ?(categories = default_metadata.categories)
+      ?(purpose = Internal_job)
+      ()
   =
   { loc; annots; name; categories; purpose }
 ;;
@@ -221,10 +217,10 @@ let io_to_redirection_path (kind : Io.kind) =
 ;;
 
 let command_line_enclosers
-  ~dir
-  ~(stdout_to : Io.output Io.t)
-  ~(stderr_to : Io.output Io.t)
-  ~(stdin_from : Io.input Io.t)
+      ~dir
+      ~(stdout_to : Io.output Io.t)
+      ~(stderr_to : Io.output Io.t)
+      ~(stdin_from : Io.input Io.t)
   =
   let quote fn = String.quote_for_shell (Path.to_string fn) in
   let prefix, suffix =
@@ -390,13 +386,18 @@ end = struct
              split_paths ("(internal)" :: targets_acc) (add_ctx ctx ctxs_acc) rest)
       in
       let target_names, contexts =
-        let file_targets, directory_targets =
+        let targets =
           match targets with
-          | None -> [], []
+          | None -> []
           | Some targets ->
-            Path.Build.Set.to_list targets.files, Path.Build.Set.to_list targets.dirs
+            Targets.Validated.fold
+              targets
+              ~init:[]
+              ~file:(fun path acc -> path :: acc)
+              ~dir:(fun dir acc -> dir :: acc)
+            |> List.rev
         in
-        split_paths [] Context_name.Set.empty (file_targets @ directory_targets)
+        split_paths [] Context_name.Set.empty targets
       in
       let targets =
         List.map target_names ~f:Filename.split_extension_after_dot
@@ -424,19 +425,19 @@ end = struct
   ;;
 
   let progname_and_purpose ~tag ~prog ~purpose =
-    let open Pp.O in
-    let progname = sprintf "%12s" (Fancy.short_prog_name_of_prog prog) in
-    Pp.tag tag (Pp.verbatim progname) ++ Pp.char ' ' ++ pp_purpose purpose
+    User_message.aligned_message
+      ~left:(tag, Fancy.short_prog_name_of_prog prog)
+      ~right:(pp_purpose purpose)
   ;;
 
   let pp_ok = progname_and_purpose ~tag:Ok
 
   let pp_error
-    ~prog
-    ~purpose
-    ~has_unexpected_stdout
-    ~has_unexpected_stderr
-    ~(error : Exit_status.error)
+        ~prog
+        ~purpose
+        ~has_unexpected_stdout
+        ~has_unexpected_stderr
+        ~(error : Exit_status.error)
     =
     let open Pp.O in
     let msg =
@@ -524,10 +525,9 @@ end = struct
       Has_output { with_color; without_color; has_embedded_location }
   ;;
 
-  let get_loc_and_annots ~dir ~metadata ~output =
+  let get_loc_annots_and_dir ~dir ~metadata ~output =
     let { loc; annots; _ } = metadata in
     let dir = Option.value dir ~default:Path.root in
-    let annots = User_message.Annots.set annots with_directory_annot dir in
     let annots =
       match output with
       | No_output -> annots
@@ -542,14 +542,15 @@ end = struct
           | errors -> User_message.Annots.set annots Compound_user_error.annot errors)
         else annots
     in
-    loc, annots
+    loc, annots, dir
   ;;
 
-  let fail ~loc ~annots paragraphs =
+  let fail ?dir ~loc ~annots paragraphs =
     (* We don't use [User_error.make] as it would add the "Error: " prefix. We
        don't need this prefix as it is already included in the output of the
        command. *)
-    raise (User_error.E (User_message.make ?loc ~annots paragraphs))
+    let dir = Option.map ~f:Path.to_string dir in
+    raise (User_error.E (User_message.make ?dir ?loc ~annots paragraphs))
   ;;
 
   let verbose t ~id ~metadata ~output ~command_line ~dir =
@@ -574,8 +575,9 @@ end = struct
         | Failed n -> sprintf "exited with code %d" n
         | Signaled signame -> sprintf "got signal %s" (Signal.name signame)
       in
-      let loc, annots = get_loc_and_annots ~dir ~metadata ~output in
+      let loc, annots, dir = get_loc_annots_and_dir ~dir ~metadata ~output in
       fail
+        ~dir
         ~loc
         ~annots
         ((Pp.tag User_message.Style.Kwd (Pp.verbatim "Command")
@@ -589,20 +591,21 @@ end = struct
   ;;
 
   let non_verbose
-    t
-    ~(verbosity : Display.t)
-    ~metadata
-    ~output
-    ~prog
-    ~command_line
-    ~dir
-    ~has_unexpected_stdout
-    ~has_unexpected_stderr
+        t
+        ~(verbosity : Display.t)
+        ~metadata
+        ~output
+        ~prog
+        ~command_line
+        ~dir
+        ~has_unexpected_stdout
+        ~has_unexpected_stderr
     =
     let output = parse_output output in
     let show_command =
       !Clflags.always_show_command_line
-      || (* We want to show command lines in the CI, but not when running inside
+      ||
+      (* We want to show command lines in the CI, but not when running inside
             dune. Otherwise tests would yield different result whether they are
             executed locally or in the CI. *)
       (Execution_env.inside_ci && not Execution_env.inside_dune)
@@ -630,7 +633,7 @@ end = struct
       then Console.print_user_message (User_message.make paragraphs);
       n
     | Error error ->
-      let loc, annots = get_loc_and_annots ~dir ~metadata ~output in
+      let loc, annots, dir = get_loc_annots_and_dir ~dir ~metadata ~output in
       let paragraphs =
         match verbosity with
         | Short ->
@@ -649,11 +652,19 @@ end = struct
                (* If the command has no output, we need to say something.
                   Otherwise it's not clear what's going on. *)
                (match error with
-                | Failed n -> [ Pp.textf "Command exited with code %d." n ]
+                | Failed exitcode ->
+                  (* When debugging segfaults, 0xc0000005 is a slightly easier
+                     random number to pattern match and 🤦 than -1073741819! *)
+                  let exitcode =
+                    if Sys.win32 && exitcode < 0
+                    then Printf.sprintf "0x%08lx" (Int32.of_int exitcode)
+                    else string_of_int exitcode
+                  in
+                  [ Pp.textf "Command exited with code %s." exitcode ]
                 | Signaled signame ->
                   [ Pp.textf "Command got signal %s." (Signal.name signame) ]))
       in
-      fail ~loc ~annots paragraphs
+      fail ~dir ~loc ~annots paragraphs
   ;;
 end
 
@@ -744,17 +755,17 @@ module Result = struct
   ;;
 
   let make
-    ({ stdout_on_success
-     ; stderr_on_success
-     ; stdout_limit
-     ; stderr_limit
-     ; stdout
-     ; stderr
-     ; _
-     } :
-      process)
-    (process_info : Proc.Process_info.t)
-    fail_mode
+        ({ stdout_on_success
+         ; stderr_on_success
+         ; stdout_limit
+         ; stderr_limit
+         ; stdout
+         ; stderr
+         ; _
+         } :
+          process)
+        (process_info : Proc.Process_info.t)
+        fail_mode
     =
     let stdout = Out.make stdout ~on_success:stdout_on_success ~limit:stdout_limit in
     let stderr = Out.make stderr ~on_success:stderr_on_success ~limit:stderr_limit in
@@ -775,17 +786,17 @@ module Result = struct
 end
 
 let report_process_finished
-  stats
-  ~metadata
-  ~dir
-  ~prog
-  ~pid
-  ~args
-  ~started_at
-  ~exit_status
-  ~stdout
-  ~stderr
-  (times : Proc.Times.t)
+      stats
+      ~metadata
+      ~dir
+      ~prog
+      ~pid
+      ~args
+      ~started_at
+      ~exit_status
+      ~stdout
+      ~stderr
+      (times : Proc.Times.t)
   =
   let common =
     let name =
@@ -809,15 +820,7 @@ let report_process_finished
         match metadata.purpose with
         | Internal_job -> []
         | Build_job None -> []
-        | Build_job (Some { files; dirs }) ->
-          let mkset s xs =
-            match
-              Path.Build.Set.to_list_map ~f:(fun x -> `String (Path.Build.to_string x)) xs
-            with
-            | [] -> []
-            | xs -> [ s, `List xs ]
-          in
-          [ "targets", `Assoc (mkset "files" files @ mkset "dirs" dirs) ]
+        | Build_job (Some targets) -> Targets.Validated.to_trace_args targets
       in
       let exit =
         match exit_status with
@@ -856,19 +859,19 @@ let await { response_file; pid; _ } =
   let+ process_info, termination_reason =
     Scheduler.wait_for_build_process pid ~is_process_group_leader:true
   in
-  Option.iter response_file ~f:Path.unlink;
+  Option.iter response_file ~f:Path.unlink_exn;
   process_info, termination_reason
 ;;
 
 let spawn
-  ?dir
-  ?(env = Env.initial)
-  ~(stdout : _ Io.t)
-  ~(stderr : _ Io.t)
-  ~(stdin : _ Io.t)
-  ~prog
-  ~args
-  ()
+      ?dir
+      ?(env = Env.initial)
+      ~(stdout : _ Io.t)
+      ~(stderr : _ Io.t)
+      ~(stdin : _ Io.t)
+      ~prog
+      ~args
+      ()
   =
   let stdout_on_success = Io.output_on_success stdout
   and stderr_on_success = Io.output_on_success stderr in
@@ -972,16 +975,16 @@ let spawn
 ;;
 
 let run_internal
-  ?dir
-  ~(display : Display.t)
-  ?(stdout_to = Io.stdout)
-  ?(stderr_to = Io.stderr)
-  ?(stdin_from = Io.null In)
-  ?env
-  ?(metadata = default_metadata)
-  fail_mode
-  prog
-  args
+      ?dir
+      ~(display : Display.t)
+      ?(stdout_to = Io.stdout)
+      ?(stderr_to = Io.stderr)
+      ?(stdin_from = Io.null In)
+      ?env
+      ?(metadata = default_metadata)
+      fail_mode
+      prog
+      args
   =
   Scheduler.with_job_slot (fun _cancel (config : Scheduler.Config.t) ->
     let dir =
@@ -1109,16 +1112,16 @@ let run ?dir ~display ?stdout_to ?stderr_to ?stdin_from ?env ?metadata fail_mode
 ;;
 
 let run_with_times
-  ?dir
-  ~display
-  ?stdout_to
-  ?stderr_to
-  ?stdin_from
-  ?env
-  ?metadata
-  fail_mode
-  prog
-  args
+      ?dir
+      ~display
+      ?stdout_to
+      ?stderr_to
+      ?stdin_from
+      ?env
+      ?metadata
+      fail_mode
+      prog
+      args
   =
   let+ code, times =
     run_internal
@@ -1137,16 +1140,16 @@ let run_with_times
 ;;
 
 let run_capture_gen
-  ?dir
-  ~display
-  ?stderr_to
-  ?stdin_from
-  ?env
-  ?metadata
-  fail_mode
-  prog
-  args
-  ~f
+      ?dir
+      ~display
+      ?stderr_to
+      ?stdin_from
+      ?env
+      ?metadata
+      fail_mode
+      prog
+      args
+      ~f
   =
   let fn = Temp.create File ~prefix:"dune" ~suffix:"output" in
   let+ run =
@@ -1174,15 +1177,15 @@ let run_capture_lines = run_capture_gen ~f:Stdune.Io.lines_of_file
 let run_capture_zero_separated = run_capture_gen ~f:Stdune.Io.zero_strings_of_file
 
 let run_capture_line
-  ?dir
-  ~display
-  ?stderr_to
-  ?stdin_from
-  ?env
-  ?metadata
-  fail_mode
-  prog
-  args
+      ?dir
+      ~display
+      ?stderr_to
+      ?stdin_from
+      ?env
+      ?metadata
+      fail_mode
+      prog
+      args
   =
   run_capture_gen
     ?dir
