@@ -5,14 +5,16 @@ module Pkg_info = struct
     { name : Package_name.t
     ; version : Package_version.t
     ; dev : bool
+    ; avoid : bool
     ; source : Source.t option
     ; extra_sources : (Path.Local.t * Source.t) list
     }
 
-  let equal { name; version; dev; source; extra_sources } t =
+  let equal { name; version; dev; avoid; source; extra_sources } t =
     Package_name.equal name t.name
     && Package_version.equal version t.version
     && Bool.equal dev t.dev
+    && Bool.equal avoid t.avoid
     && Option.equal Source.equal source t.source
     && List.equal
          (Tuple.T2.equal Path.Local.equal Source.equal)
@@ -29,17 +31,27 @@ module Pkg_info = struct
     }
   ;;
 
-  let to_dyn { name; version; dev; source; extra_sources } =
+  let to_dyn { name; version; dev; avoid; source; extra_sources } =
     Dyn.record
       [ "name", Package_name.to_dyn name
       ; "version", Package_version.to_dyn version
       ; "dev", Dyn.bool dev
+      ; "avoid", Dyn.bool avoid
       ; "source", Dyn.option Source.to_dyn source
       ; "extra_sources", Dyn.list (Dyn.pair Path.Local.to_dyn Source.to_dyn) extra_sources
       ]
   ;;
 
-  let default_version = Package_version.of_string "dev"
+  let default_version = Package_version.dev
+
+  let variables t =
+    let module Variable = OpamVariable in
+    Package_variable_name.Map.of_list_exn
+      [ Package_variable_name.name, Variable.S (Package_name.to_string t.name)
+      ; Package_variable_name.version, S (Package_version.to_string t.version)
+      ; Package_variable_name.dev, B t.dev
+      ]
+  ;;
 end
 
 module Build_command = struct
@@ -96,15 +108,17 @@ module Pkg = struct
     { build_command : Build_command.t option
     ; install_command : Action.t option
     ; depends : (Loc.t * Package_name.t) list
+    ; depexts : string list
     ; info : Pkg_info.t
     ; exported_env : String_with_vars.t Action.Env_update.t list
     }
 
-  let equal { build_command; install_command; depends; info; exported_env } t =
+  let equal { build_command; install_command; depends; depexts; info; exported_env } t =
     Option.equal Build_command.equal build_command t.build_command
     (* CR-rgrinberg: why do we ignore locations? *)
     && Option.equal Action.equal_no_locs install_command t.install_command
     && List.equal (Tuple.T2.equal Loc.equal Package_name.equal) depends t.depends
+    && List.equal String.equal depexts t.depexts
     && Pkg_info.equal info t.info
     && List.equal
          (Action.Env_update.equal String_with_vars.equal)
@@ -112,21 +126,24 @@ module Pkg = struct
          t.exported_env
   ;;
 
-  let remove_locs { build_command; install_command; depends; info; exported_env } =
+  let remove_locs { build_command; install_command; depends; depexts; info; exported_env }
+    =
     { info = Pkg_info.remove_locs info
     ; exported_env =
         List.map exported_env ~f:(Action.Env_update.map ~f:String_with_vars.remove_locs)
     ; depends = List.map depends ~f:(fun (_, pkg) -> Loc.none, pkg)
+    ; depexts
     ; build_command = Option.map build_command ~f:Build_command.remove_locs
     ; install_command = Option.map install_command ~f:Action.remove_locs
     }
   ;;
 
-  let to_dyn { build_command; install_command; depends; info; exported_env } =
+  let to_dyn { build_command; install_command; depends; depexts; info; exported_env } =
     Dyn.record
       [ "build_command", Dyn.option Build_command.to_dyn build_command
       ; "install_command", Dyn.option Action.to_dyn install_command
       ; "depends", Dyn.list (Dyn.pair Loc.to_dyn_hum Package_name.to_dyn) depends
+      ; "depexts", Dyn.list String.to_dyn depexts
       ; "info", Pkg_info.to_dyn info
       ; ( "exported_env"
         , Dyn.list (Action.Env_update.to_dyn String_with_vars.to_dyn) exported_env )
@@ -148,8 +165,10 @@ module Pkg = struct
     let version = "version"
     let install = "install"
     let depends = "depends"
+    let depexts = "depexts"
     let source = "source"
     let dev = "dev"
+    let avoid = "avoid"
     let exported_env = "exported_env"
     let extra_sources = "extra_sources"
   end
@@ -163,8 +182,10 @@ module Pkg = struct
        and+ build_command = Build_command.decode
        and+ depends =
          field ~default:[] Fields.depends (repeat (located Package_name.decode))
+       and+ depexts = field ~default:[] Fields.depexts (repeat string)
        and+ source = field_o Fields.source Source.decode
        and+ dev = field_b Fields.dev
+       and+ avoid = field_b Fields.avoid
        and+ exported_env =
          field Fields.exported_env ~default:[] (repeat Action.Env_update.decode)
        and+ extra_sources =
@@ -185,9 +206,9 @@ module Pkg = struct
            let extra_sources =
              List.map extra_sources ~f:(fun (path, source) -> path, make_source source)
            in
-           { Pkg_info.name; version; dev; source; extra_sources }
+           { Pkg_info.name; version; dev; avoid; source; extra_sources }
          in
-         { build_command; depends; install_command; info; exported_env }
+         { build_command; depends; depexts; install_command; info; exported_env }
   ;;
 
   let encode_extra_source (local, source) : Dune_sexp.t =
@@ -198,12 +219,13 @@ module Pkg = struct
   ;;
 
   let encode
-    { build_command
-    ; install_command
-    ; depends
-    ; info = { Pkg_info.name = _; extra_sources; version; dev; source }
-    ; exported_env
-    }
+        { build_command
+        ; install_command
+        ; depends
+        ; depexts
+        ; info = { Pkg_info.name = _; extra_sources; version; dev; avoid; source }
+        ; exported_env
+        }
     =
     let open Encoder in
     record_fields
@@ -211,8 +233,10 @@ module Pkg = struct
       ; field_o Fields.install Action.encode install_command
       ; Build_command.encode build_command
       ; field_l Fields.depends Package_name.encode (List.map depends ~f:snd)
+      ; field_l Fields.depexts string depexts
       ; field_o Fields.source Source.encode source
       ; field_b Fields.dev dev
+      ; field_b Fields.avoid avoid
       ; field_l Fields.exported_env Action.Env_update.encode exported_env
       ; field_l Fields.extra_sources encode_extra_source extra_sources
       ]
@@ -285,8 +309,14 @@ let remove_locs t =
 ;;
 
 let equal
-  { version; dependency_hash; packages; ocaml; repos; expanded_solver_variable_bindings }
-  t
+      { version
+      ; dependency_hash
+      ; packages
+      ; ocaml
+      ; repos
+      ; expanded_solver_variable_bindings
+      }
+      t
   =
   Syntax.Version.equal version t.version
   && Option.equal
@@ -302,7 +332,13 @@ let equal
 ;;
 
 let to_dyn
-  { version; dependency_hash; packages; ocaml; repos; expanded_solver_variable_bindings }
+      { version
+      ; dependency_hash
+      ; packages
+      ; ocaml
+      ; repos
+      ; expanded_solver_variable_bindings
+      }
   =
   Dyn.record
     [ "version", Syntax.Version.to_dyn version
@@ -333,7 +369,11 @@ let validate_packages packages =
     Package_name.Map.values packages
     |> List.concat_map ~f:(fun (dependant_package : Pkg.t) ->
       List.filter_map dependant_package.depends ~f:(fun (loc, dependency) ->
-        if Package_name.Map.mem packages dependency
+        (* CR-someday rgrinberg: do we need the dune check? aren't
+           we supposed to filter these upfront? *)
+        if
+          Package_name.Map.mem packages dependency
+          || Package_name.equal dependency Dune_dep.name
         then None
         else Some { dependant_package; dependency; loc }))
   in
@@ -343,11 +383,11 @@ let validate_packages packages =
 ;;
 
 let create_latest_version
-  packages
-  ~local_packages
-  ~ocaml
-  ~repos
-  ~expanded_solver_variable_bindings
+      packages
+      ~local_packages
+      ~ocaml
+      ~repos
+      ~expanded_solver_variable_bindings
   =
   (match validate_packages packages with
    | Ok () -> ()
@@ -361,8 +401,9 @@ let create_latest_version
      |> Code_error.raise "Invalid package table");
   let version = Syntax.greatest_supported_version_exn Dune_lang.Pkg.syntax in
   let dependency_hash =
-    Local_package.(
-      For_solver.list_non_local_dependency_set local_packages |> Dependency_set.hash)
+    local_packages
+    |> Local_package.For_solver.non_local_dependencies
+    |> Local_package.Dependency_hash.of_dependency_formula
     |> Option.map ~f:(fun dependency_hash -> Loc.none, dependency_hash)
   in
   let complete, used =
@@ -382,6 +423,14 @@ let create_latest_version
   }
 ;;
 
+let dev_tools_path = Path.Source.(relative root "dev-tools.locks")
+
+let dev_tool_lock_dir_path dev_tool =
+  Path.Source.relative
+    dev_tools_path
+    (Package_name.to_string (Dev_tool.package_name dev_tool))
+;;
+
 let default_path = Path.Source.(relative root "dune.lock")
 let metadata_filename = "lock.dune"
 
@@ -390,13 +439,13 @@ module Metadata = Dune_sexp.Versioned_file.Make (Unit)
 let () = Metadata.Lang.register Dune_lang.Pkg.syntax ()
 
 let encode_metadata
-  { version
-  ; dependency_hash
-  ; ocaml
-  ; repos
-  ; packages = _
-  ; expanded_solver_variable_bindings
-  }
+      { version
+      ; dependency_hash
+      ; ocaml
+      ; repos
+      ; packages = _
+      ; expanded_solver_variable_bindings
+      }
   =
   let open Encoder in
   let base =
@@ -488,6 +537,36 @@ module Write_disk = struct
     | Ok _ -> Error `Not_directory
   ;;
 
+  let raise_user_error_on_check_existance path e =
+    let error_reason =
+      match e with
+      | `Unreadable ->
+        Pp.textf "Unable to read lock directory (%s)" (Path.to_string_maybe_quoted path)
+      | `Not_directory ->
+        Pp.textf
+          "Specified lock dir path (%s) is not a directory"
+          (Path.to_string_maybe_quoted path)
+      | `No_metadata_file ->
+        Pp.textf "Specified lock dir lacks metadata file (%s)" metadata_filename
+      | `Failed_to_parse_metadata (path, exn) ->
+        Pp.concat
+          ~sep:Pp.cut
+          [ Pp.textf
+              "Unable to parse lock directory metadata file (%s):"
+              (Path.to_string_maybe_quoted path)
+            |> Pp.hovbox
+          ; Exn.pp exn |> Pp.hovbox
+          ]
+        |> Pp.vbox
+    in
+    User_error.raise
+      [ Pp.textf
+          "Refusing to regenerate lock directory %s"
+          (Path.to_string_maybe_quoted path)
+      ; error_reason
+      ]
+  ;;
+
   (* Removes the existing lock directory at the specified path if it exists and
      is a valid lock directory. Checks the validity of the existing lockdir (if
      any) and raises if it's invalid before constructing the returned thunk, so
@@ -497,40 +576,53 @@ module Write_disk = struct
     match check_existing_lock_dir path with
     | Ok `Non_existant -> Fun.const ()
     | Ok `Is_existing_lock_dir -> fun () -> Path.rm_rf path
-    | Error e ->
+    | Error e -> raise_user_error_on_check_existance path e
+  ;;
+
+  (* Does the same checks as [safely_remove_lock_dir_if_exists_thunk] but it raises an
+     error if the lock dir already exists. [dst] is the new file name *)
+  let safely_rename_lock_dir_thunk ~dst src =
+    match check_existing_lock_dir src, check_existing_lock_dir dst with
+    | Ok `Is_existing_lock_dir, Ok `Non_existant -> fun () -> Path.rename src dst
+    | Ok `Non_existant, Ok `Non_existant -> Fun.const ()
+    | _, Ok `Is_existing_lock_dir ->
       let error_reason_pp =
-        match e with
-        | `Unreadable -> Pp.text "Unable to read lock directory"
-        | `Not_directory -> Pp.text "Specified lock dir path is not a directory"
-        | `No_metadata_file ->
-          Pp.textf "Specified lock dir lacks metadata file (%s)" metadata_filename
-        | `Failed_to_parse_metadata (path, exn) ->
-          Pp.concat
-            ~sep:Pp.cut
-            [ Pp.textf
-                "Unable to parse lock directory metadata file (%s):"
-                (Path.to_string_maybe_quoted path)
-              |> Pp.hovbox
-            ; Exn.pp exn |> Pp.hovbox
-            ]
-          |> Pp.vbox
+        Pp.textf
+          "Directory %s already exists: can't rename safely"
+          (Path.to_string_maybe_quoted src)
       in
       User_error.raise
         [ Pp.textf
             "Refusing to regenerate lock directory %s"
-            (Path.to_string_maybe_quoted path)
+            (Path.to_string_maybe_quoted src)
         ; error_reason_pp
         ]
+    | Error e, _ -> raise_user_error_on_check_existance src e
+    | _, Error e -> raise_user_error_on_check_existance dst e
   ;;
 
   type t = unit -> unit
 
-  let prepare ~lock_dir_path:lock_dir_path_src ~files lock_dir =
-    let lock_dir_path = Path.source lock_dir_path_src in
-    let remove_dir_if_exists = safely_remove_lock_dir_if_exists_thunk lock_dir_path in
-    fun () ->
-      remove_dir_if_exists ();
-      Path.mkdir_p lock_dir_path;
+  let prepare
+        ~lock_dir_path:lock_dir_path_src
+        ~(files : File_entry.t Package_name.Map.Multi.t)
+        lock_dir
+    =
+    let lock_dir_hidden_src =
+      (* The original lockdir path with the lockdir renamed to begin with a ".". *)
+      let hidden_basename = sprintf ".%s" (Path.Source.basename lock_dir_path_src) in
+      Path.Source.relative (Path.Source.parent_exn lock_dir_path_src) hidden_basename
+    in
+    let lock_dir_hidden_src = Path.source lock_dir_hidden_src in
+    let lock_dir_path_external = Path.source lock_dir_path_src in
+    let remove_hidden_dir_if_exists () =
+      safely_remove_lock_dir_if_exists_thunk lock_dir_hidden_src ()
+    in
+    let rename_old_lock_dir_to_hidden =
+      safely_rename_lock_dir_thunk ~dst:lock_dir_hidden_src lock_dir_path_external
+    in
+    let build lock_dir_path =
+      let lock_dir_path = Result.ok_exn lock_dir_path in
       file_contents_by_path lock_dir
       |> List.iter ~f:(fun (path_within_lock_dir, contents) ->
         let path = Path.relative lock_dir_path path_within_lock_dir in
@@ -545,7 +637,7 @@ module Write_disk = struct
         Format.asprintf "%a" Pp.to_fmt pp |> Io.write_file path;
         Package_name.Map.iteri files ~f:(fun package_name files ->
           let files_dir =
-            Pkg.files_dir package_name ~lock_dir:lock_dir_path_src |> Path.source
+            Path.relative lock_dir_path (Package_name.to_string package_name ^ ".files")
           in
           Path.mkdir_p files_dir;
           List.iter files ~f:(fun { File_entry.original; local_file } ->
@@ -553,7 +645,19 @@ module Write_disk = struct
             Path.mkdir_p (Path.parent_exn dst);
             match original with
             | Path src -> Io.copy_file ~src ~dst ()
-            | Content content -> Io.write_file dst content)))
+            | Content content -> Io.write_file dst content)));
+      rename_old_lock_dir_to_hidden ();
+      safely_rename_lock_dir_thunk ~dst:lock_dir_path_external lock_dir_path ();
+      remove_hidden_dir_if_exists ()
+    in
+    match Path.(parent (source lock_dir_path_src)) with
+    | Some parent_dir ->
+      fun () ->
+        Path.mkdir_p parent_dir;
+        Temp.with_temp_dir ~parent_dir ~prefix:"dune" ~suffix:"lock" ~f:build
+    | None ->
+      User_error.raise
+        [ Pp.textf "Temporary directory can't be created by deriving the lock dir path" ]
   ;;
 
   let commit t = t ()
@@ -624,32 +728,35 @@ struct
     let open Io.O in
     Io.stats_kind lock_dir_path
     >>| function
-    | Ok S_DIR -> ()
+    | Ok S_DIR -> Ok ()
     | Error (Unix.ENOENT, _, _) ->
-      User_error.raise
-        ~hints:
-          [ Pp.concat
-              ~sep:Pp.space
-              [ Pp.text "Run"
-              ; User_message.command "dune pkg lock"
-              ; Pp.text "to generate it."
-              ]
-            |> Pp.hovbox
-          ]
-        [ Pp.textf "%s does not exist." (Path.Source.to_string lock_dir_path) ]
+      Error
+        (User_error.make
+           ~hints:
+             [ Pp.concat
+                 ~sep:Pp.space
+                 [ Pp.text "Run"
+                 ; User_message.command "dune pkg lock"
+                 ; Pp.text "to generate it."
+                 ]
+               |> Pp.hovbox
+             ]
+           [ Pp.textf "%s does not exist." (Path.Source.to_string lock_dir_path) ])
     | Error e ->
-      User_error.raise
-        [ Pp.textf "%s is not accessible" (Path.Source.to_string lock_dir_path)
-        ; Pp.textf "reason: %s" (Unix_error.Detailed.to_string_hum e)
-        ]
+      Error
+        (User_error.make
+           [ Pp.textf "%s is not accessible" (Path.Source.to_string lock_dir_path)
+           ; Pp.textf "reason: %s" (Unix_error.Detailed.to_string_hum e)
+           ])
     | _ ->
-      User_error.raise
-        [ Pp.textf "%s is not a directory." (Path.Source.to_string lock_dir_path) ]
+      Error
+        (User_error.make
+           [ Pp.textf "%s is not a directory." (Path.Source.to_string lock_dir_path) ])
   ;;
 
   let check_packages packages ~lock_dir_path =
     match validate_packages packages with
-    | Ok () -> ()
+    | Ok () -> Ok ()
     | Error (`Missing_dependencies missing_dependencies) ->
       List.iter missing_dependencies ~f:(fun { dependant_package; dependency; loc } ->
         User_message.prerr
@@ -663,50 +770,60 @@ struct
                  (Package_name.to_string dependency)
                  (Path.Source.to_string_maybe_quoted lock_dir_path)
              ]));
-      User_error.raise
-        ~hints:
-          [ Pp.concat
-              ~sep:Pp.space
-              [ Pp.text
-                  "This could indicate that the lockdir is corrupted. Delete it and then \
-                   regenerate it by running:"
-              ; User_message.command "dune pkg lock"
-              ]
-          ]
-        [ Pp.textf
-            "At least one package dependency is itself not present as a package in the \
-             lockdir %s."
-            (Path.Source.to_string_maybe_quoted lock_dir_path)
-        ]
+      Error
+        (User_error.make
+           ~hints:
+             [ Pp.concat
+                 ~sep:Pp.space
+                 [ Pp.text
+                     "This could indicate that the lockdir is corrupted. Delete it and \
+                      then regenerate it by running:"
+                 ; User_message.command "dune pkg lock"
+                 ]
+             ]
+           [ Pp.textf
+               "At least one package dependency is itself not present as a package in \
+                the lockdir %s."
+               (Path.Source.to_string_maybe_quoted lock_dir_path)
+           ])
   ;;
 
   let load lock_dir_path =
     let open Io.O in
-    let* () = check_path lock_dir_path in
-    let* version, dependency_hash, ocaml, repos, expanded_solver_variable_bindings =
-      load_metadata (Path.Source.relative lock_dir_path metadata_filename)
-    in
-    let+ packages =
-      Io.readdir_with_kinds lock_dir_path
-      >>| List.filter_map ~f:(fun (name, (kind : Unix.file_kind)) ->
-        match kind with
-        | S_REG -> Package_filename.to_package_name name |> Result.to_option
-        | _ ->
-          (* TODO *)
-          None)
-      >>= Io.parallel_map ~f:(fun package_name ->
-        let+ pkg = load_pkg ~version ~lock_dir_path package_name in
-        package_name, pkg)
-      >>| Package_name.Map.of_list_exn
-    in
-    check_packages packages ~lock_dir_path;
-    { version
-    ; dependency_hash
-    ; packages
-    ; ocaml
-    ; repos
-    ; expanded_solver_variable_bindings
-    }
+    let* result = check_path lock_dir_path in
+    match result with
+    | Error e -> Io.return (Error e)
+    | Ok () ->
+      let* version, dependency_hash, ocaml, repos, expanded_solver_variable_bindings =
+        load_metadata (Path.Source.relative lock_dir_path metadata_filename)
+      in
+      let+ packages =
+        Io.readdir_with_kinds lock_dir_path
+        >>| List.filter_map ~f:(fun (name, (kind : Unix.file_kind)) ->
+          match kind with
+          | S_REG -> Package_filename.to_package_name name |> Result.to_option
+          | _ ->
+            (* TODO *)
+            None)
+        >>= Io.parallel_map ~f:(fun package_name ->
+          let+ pkg = load_pkg ~version ~lock_dir_path package_name in
+          package_name, pkg)
+        >>| Package_name.Map.of_list_exn
+      in
+      check_packages packages ~lock_dir_path
+      |> Result.map ~f:(fun () ->
+        { version
+        ; dependency_hash
+        ; packages
+        ; ocaml
+        ; repos
+        ; expanded_solver_variable_bindings
+        })
+  ;;
+
+  let load_exn lock_dir_path =
+    let open Io.O in
+    load lock_dir_path >>| User_error.ok_exn
   ;;
 end
 
@@ -731,6 +848,7 @@ module Load_immediate = Make_load (struct
   end)
 
 let read_disk = Load_immediate.load
+let read_disk_exn = Load_immediate.load_exn
 
 let transitive_dependency_closure t start =
   let missing_packages =
