@@ -106,8 +106,9 @@ let select_native_mode ~sctx ~dir (buildable : Coq_stanza.Buildable.t) =
   | Some x ->
     Memo.return
     @@
-    if buildable.coq_lang_version < (0, 7)
-       && Super_context.context sctx |> Context.profile |> Profile.is_dev
+    if
+      buildable.coq_lang_version < (0, 7)
+      && Super_context.context sctx |> Context.profile |> Profile.is_dev
     then Coq_mode.VoOnly
     else x
   | None ->
@@ -141,6 +142,15 @@ let coq_env =
             x.coq_flags
           in
           Expander.expand_and_eval_set expander (Coq_env.flags config.coq) ~standard
+        and+ coqdep_flags =
+          let standard =
+            let+ x = Action_builder.of_memo_join parent in
+            x.coqdep_flags
+          in
+          Expander.expand_and_eval_set
+            expander
+            (Coq_env.coqdep_flags config.coq)
+            ~standard
         and+ coqdoc_flags =
           let standard =
             let+ x = Action_builder.of_memo_join parent in
@@ -151,7 +161,7 @@ let coq_env =
             (Coq_env.coqdoc_flags config.coq)
             ~standard
         in
-        { Coq_flags.coq_flags; coqdoc_flags })
+        { Coq_flags.coq_flags; coqdep_flags; coqdoc_flags })
   in
   fun ~dir ->
     (let* () = Memo.return () in
@@ -159,12 +169,31 @@ let coq_env =
     |> Action_builder.of_memo_join
 ;;
 
-let coq_flags ~dir ~stanza_flags ~expander =
+let coq_flags ~dir ~stanza_flags ~per_file_flags ~expander =
+  let standard, flags_to_expand =
+    match per_file_flags with
+    | None ->
+      ( Action_builder.map ~f:(fun { Coq_flags.coq_flags; _ } -> coq_flags) (coq_env ~dir)
+      , stanza_flags )
+    | Some per_file_flags ->
+      ( Action_builder.bind
+          ~f:(fun { Coq_flags.coq_flags; _ } ->
+            let standard = Action_builder.return coq_flags in
+            Expander.expand_and_eval_set expander stanza_flags ~standard)
+          (coq_env ~dir)
+      , per_file_flags )
+  in
+  Expander.expand_and_eval_set expander flags_to_expand ~standard
+;;
+
+let coqdep_flags ~dir ~stanza_coqdep_flags ~expander =
   Expander.expand_and_eval_set
     expander
-    stanza_flags
+    stanza_coqdep_flags
     ~standard:
-      (Action_builder.map ~f:(fun { Coq_flags.coq_flags; _ } -> coq_flags) (coq_env ~dir))
+      (Action_builder.map
+         ~f:(fun { Coq_flags.coqdep_flags; _ } -> coqdep_flags)
+         (coq_env ~dir))
 ;;
 
 let coqdoc_flags ~dir ~stanza_coqdoc_flags ~expander =
@@ -197,7 +226,8 @@ module Bootstrap : sig
     | No_stdlib
     (** We are in >= 0.8, however the user set stdlib = no
         , or we are compiling the prelude *)
-    | Stdlib of Coq_lib.t (** Regular case in >= 0.8 (or in < 0.8
+    | Stdlib of Coq_lib.t
+    (** Regular case in >= 0.8 (or in < 0.8
                               (boot) was used *)
 
   val empty : t
@@ -217,7 +247,8 @@ end = struct
     | No_stdlib
     (** We are in >= 0.8, however the user set stdlib = no
         , or we are compiling the prelude *)
-    | Stdlib of Coq_lib.t (** Regular case in >= 0.8 (or in < 0.8
+    | Stdlib of Coq_lib.t
+    (** Regular case in >= 0.8 (or in < 0.8
                               (boot) was used *)
 
   (* For empty set of modules, we return Prelude which is kinda
@@ -282,9 +313,7 @@ end = struct
   ;;
 end
 
-let coqc_file_flags ~dir ~theories_deps ~wrapper_name ~boot_type ~ml_flags
-  : _ Command.Args.t list
-  =
+let coqc_file_flags ~dir ~theories_deps ~wrapper_name ~ml_flags : _ Command.Args.t list =
   let file_flags : _ Command.Args.t list =
     [ Dyn (Resolve.Memo.read ml_flags)
     ; theories_flags ~theories_deps
@@ -293,8 +322,7 @@ let coqc_file_flags ~dir ~theories_deps ~wrapper_name ~boot_type ~ml_flags
     ; A wrapper_name
     ]
   in
-  let boot_flags = Resolve.Memo.read boot_type |> Action_builder.map ~f:Bootstrap.flags in
-  [ Dyn boot_flags; S file_flags ]
+  [ S file_flags ]
 ;;
 
 let native_includes ~dir =
@@ -430,10 +458,10 @@ let ml_pack_and_meta_rule ~context ~all_libs (buildable : Coq_stanza.Buildable.t
 ;;
 
 let ml_flags_and_ml_pack_rule
-  ~context
-  ~lib_db
-  ~theories_deps
-  (buildable : Coq_stanza.Buildable.t)
+      ~context
+      ~lib_db
+      ~theories_deps
+      (buildable : Coq_stanza.Buildable.t)
   =
   let res =
     let open Resolve.Memo.O in
@@ -459,36 +487,35 @@ let dep_theory_file ~dir ~wrapper_name =
 ;;
 
 let setup_coqdep_for_theory_rule
-  ~scope
-  ~sctx
-  ~dir
-  ~loc
-  ~theories_deps
-  ~wrapper_name
-  ~use_stdlib
-  ~source_rule
-  ~ml_flags
-  ~mlpack_rule
-  ~coq_lang_version
-  coq_modules
+      ~sctx
+      ~dir
+      ~loc
+      ~theories_deps
+      ~wrapper_name
+      ~source_rule
+      ~ml_flags
+      ~mlpack_rule
+      ~boot_flags
+      ~stanza_coqdep_flags
+      coq_modules
   =
-  let boot_type =
-    (* If coq_modules are empty it doesn't really matter, so we take
-       the more conservative path and pass -boot, we don't care here
-       about -noinit as coqdep ignores it *)
-    match coq_modules with
-    | [] -> Resolve.Memo.return Bootstrap.empty
-    | m :: _ -> Bootstrap.make ~scope ~use_stdlib ~wrapper_name ~coq_lang_version m
-  in
   (* coqdep needs the full source + plugin's mlpack to be present :( *)
   let sources = List.rev_map ~f:Coq_module.source coq_modules in
   let file_flags =
-    [ Command.Args.S
-        (coqc_file_flags ~dir ~theories_deps ~wrapper_name ~boot_type ~ml_flags)
+    [ Command.Args.S (coqc_file_flags ~dir ~theories_deps ~wrapper_name ~ml_flags)
     ; As [ "-dyndep"; "opt"; "-vos" ]
     ; Deps sources
     ]
   in
+  let extra_coqdep_flags =
+    (* Standard flags for coqdep *)
+    let open Action_builder.O in
+    let* expander = Action_builder.of_memo @@ Super_context.expander sctx ~dir in
+    coqdep_flags ~dir ~stanza_coqdep_flags ~expander
+  in
+  let coqdep_flags =
+    Command.Args.Dyn boot_flags :: Command.Args.dyn extra_coqdep_flags :: file_flags
+  in
   let stdout_to = dep_theory_file ~dir ~wrapper_name in
   let* coqdep =
     Super_context.resolve_program_memo
@@ -507,7 +534,7 @@ let setup_coqdep_for_theory_rule
     (let open Action_builder.With_targets.O in
      Action_builder.with_no_targets mlpack_rule
      >>> Action_builder.(with_no_targets (goal source_rule))
-     >>> Command.run ~dir:(Path.build dir) ~stdout_to coqdep file_flags)
+     >>> Command.run ~dir:(Path.build dir) ~stdout_to coqdep coqdep_flags)
 ;;
 
 module Dep_map = Stdune.Map.Make (Path)
@@ -621,22 +648,23 @@ let deps_of ~dir ~boot_type ~wrapper_name ~mode coq_module =
 ;;
 
 let generic_coq_args
-  ~sctx
-  ~dir
-  ~wrapper_name
-  ~boot_type
-  ~mode
-  ~coq_prog
-  ~stanza_flags
-  ~ml_flags
-  ~theories_deps
-  ~theory_dirs
-  coq_module
+      ~sctx
+      ~dir
+      ~wrapper_name
+      ~boot_flags
+      ~per_file_flags
+      ~mode
+      ~coq_prog
+      ~stanza_flags
+      ~ml_flags
+      ~theories_deps
+      ~theory_dirs
+      coq_module
   =
   let+ coq_stanza_flags =
     let+ expander = Super_context.expander sctx ~dir in
     let coq_flags =
-      let coq_flags = coq_flags ~expander ~dir ~stanza_flags in
+      let coq_flags = coq_flags ~expander ~dir ~stanza_flags ~per_file_flags in
       (* By default we have the -q flag. We don't want to pass this to coqtop to
          allow users to load their .coqrc files for interactive development.
          Therefore we manually scrub the -q setting when passing arguments to
@@ -663,40 +691,52 @@ let generic_coq_args
     in
     coqc_native_flags ~sctx ~dir ~theories_deps ~theory_dirs ~mode
   in
-  let file_flags =
-    coqc_file_flags ~dir ~theories_deps ~wrapper_name ~boot_type ~ml_flags
-  in
+  let file_flags = coqc_file_flags ~dir ~theories_deps ~wrapper_name ~ml_flags in
   match coq_prog with
   | `Coqc ->
     [ coq_stanza_flags
     ; coq_native_flags
+    ; Dyn boot_flags
     ; S file_flags
     ; Dep (Coq_module.source coq_module)
     ]
-  | `Coqtop -> [ coq_stanza_flags; coq_native_flags; S file_flags ]
+  | `Coqtop -> [ coq_stanza_flags; coq_native_flags; Dyn boot_flags; S file_flags ]
 ;;
 
+module Per_file = struct
+  let match_ map coq_module =
+    let open Option.O in
+    let* map = map in
+    let coq_lib_name = Coq_module.name coq_module in
+    Coq_module.Name.Map.find map coq_lib_name
+  ;;
+end
+
 let setup_coqc_rule
-  ~scope
-  ~loc
-  ~dir
-  ~sctx
-  ~coqc_dir
-  ~file_targets
-  ~stanza_flags
-  ~theories_deps
-  ~mode
-  ~wrapper_name
-  ~use_stdlib
-  ~ml_flags
-  ~theory_dirs
-  ~coq_lang_version
-  coq_module
+      ~scope
+      ~loc
+      ~dir
+      ~sctx
+      ~coqc_dir
+      ~file_targets
+      ~stanza_flags
+      ~modules_flags
+      ~theories_deps
+      ~mode
+      ~wrapper_name
+      ~use_stdlib
+      ~ml_flags
+      ~theory_dirs
+      ~coq_lang_version
+      coq_module
   =
   (* Process coqdep and generate rules *)
   let boot_type =
     Bootstrap.make ~scope ~use_stdlib ~wrapper_name ~coq_lang_version coq_module
   in
+  let boot_flags = Resolve.Memo.read boot_type |> Action_builder.map ~f:Bootstrap.flags in
+  (* TODO: merge with boot_type *)
+  let per_file_flags = Per_file.match_ modules_flags coq_module in
   let* coqc = coqc ~loc ~dir ~sctx in
   let obj_files =
     Coq_module.obj_files
@@ -713,7 +753,8 @@ let setup_coqc_rule
       ~sctx
       ~dir
       ~wrapper_name
-      ~boot_type
+      ~boot_flags
+      ~per_file_flags
       ~stanza_flags
       ~ml_flags
       ~theories_deps
@@ -872,10 +913,10 @@ let theory_context ~context ~scope ~coq_lang_version ~name buildable =
 
 (* Common context for extraction, almost the same than above *)
 let extraction_context
-  ~context
-  ~scope
-  ~coq_lang_version
-  (buildable : Coq_stanza.Buildable.t)
+      ~context
+      ~scope
+      ~coq_lang_version
+      (buildable : Coq_stanza.Buildable.t)
   =
   let coq_lib_db = Scope.coq_libs scope in
   let theories_deps =
@@ -919,6 +960,7 @@ let setup_theory_rules ~sctx ~dir ~dir_contents (s : Coq_stanza.Theory.t) =
   let name = snd s.name in
   let loc = s.buildable.loc in
   let stanza_flags = s.buildable.flags in
+  let modules_flags = Option.map s.modules_flags ~f:Coq_module.Name.Map.of_list_exn in
   let* coq_dir_contents = Dir_contents.coq dir_contents in
   let theory_dirs =
     Coq_sources.directories coq_dir_contents ~name |> Path.Build.Set.of_list
@@ -938,18 +980,26 @@ let setup_theory_rules ~sctx ~dir ~dir_contents (s : Coq_stanza.Theory.t) =
   let coqc_dir = Super_context.context sctx |> Context.build_dir in
   let* mode = select_native_mode ~sctx ~dir s.buildable in
   (* First we setup the rule calling coqdep *)
+  let boot_type =
+    (* If coq_modules are empty it doesn't really matter, so we take
+       the more conservative path and pass -boot, we don't care here
+       about -noinit as coqdep ignores it *)
+    match coq_modules with
+    | [] -> Resolve.Memo.return Bootstrap.empty
+    | m :: _ -> Bootstrap.make ~scope ~use_stdlib ~wrapper_name ~coq_lang_version m
+  in
+  let boot_flags = Resolve.Memo.read boot_type |> Action_builder.map ~f:Bootstrap.flags in
   setup_coqdep_for_theory_rule
-    ~scope
     ~sctx
     ~dir
     ~loc
     ~theories_deps
     ~wrapper_name
-    ~use_stdlib
     ~source_rule
     ~ml_flags
     ~mlpack_rule
-    ~coq_lang_version
+    ~boot_flags
+    ~stanza_coqdep_flags:s.coqdep_flags
     coq_modules
   >>> Memo.parallel_iter
         coq_modules
@@ -961,6 +1011,7 @@ let setup_theory_rules ~sctx ~dir ~dir_contents (s : Coq_stanza.Theory.t) =
              ~sctx
              ~file_targets:[]
              ~stanza_flags
+             ~modules_flags
              ~coqc_dir
              ~theories_deps
              ~mode
@@ -992,11 +1043,14 @@ let coqtop_args_theory ~sctx ~dir ~dir_contents (s : Coq_stanza.Theory.t) coq_mo
   let theory_dirs =
     Coq_sources.directories coq_dir_contents ~name |> Path.Build.Set.of_list
   in
+  let boot_flags = Resolve.Memo.read boot_type |> Action_builder.map ~f:Bootstrap.flags in
+  let per_file_flags = None in
   generic_coq_args
     ~sctx
     ~dir
     ~wrapper_name
-    ~boot_type
+    ~boot_flags
+    ~per_file_flags
     ~mode
     ~coq_prog:`Coqtop
     ~stanza_flags:s.buildable.flags
@@ -1022,8 +1076,9 @@ let coq_plugins_install_rules ~scope ~package ~dst_dir (s : Coq_stanza.Theory.t)
   let rules_for_lib lib =
     let info = Lib.info lib in
     (* Don't install libraries that don't belong to this package *)
-    if let name = Package.name package in
-       Option.equal Package.Name.equal (Lib_info.package info) (Some name)
+    if
+      let name = Package.name package in
+      Option.equal Package.Name.equal (Lib_info.package info) (Some name)
     then (
       let loc = Lib_info.loc info in
       let plugins = Lib_info.plugins info in
@@ -1147,23 +1202,33 @@ let setup_extraction_rules ~sctx ~dir ~dir_contents (s : Coq_stanza.Extraction.t
     source_rule ~sctx theories_deps >>> Action_builder.path (Coq_module.source coq_module)
   in
   let* mode = select_native_mode ~sctx ~dir s.buildable in
+  let boot_type =
+    (* If coq_modules are empty it doesn't really matter, so we take
+       the more conservative path and pass -boot, we don't care here
+       about -noinit as coqdep ignores it *)
+    match [ coq_module ] with
+    | [] -> Resolve.Memo.return Bootstrap.empty
+    | m :: _ -> Bootstrap.make ~scope ~use_stdlib ~wrapper_name ~coq_lang_version m
+  in
+  let boot_flags = Resolve.Memo.read boot_type |> Action_builder.map ~f:Bootstrap.flags in
+  let modules_flags = None in
   setup_coqdep_for_theory_rule
-    ~scope
     ~sctx
     ~dir
     ~loc
     ~theories_deps
     ~wrapper_name
-    ~use_stdlib
     ~source_rule
     ~ml_flags
     ~mlpack_rule
-    ~coq_lang_version
+    ~boot_flags
+    ~stanza_coqdep_flags:Ordered_set_lang.Unexpanded.standard
     [ coq_module ]
   >>> setup_coqc_rule
         ~scope
         ~file_targets
         ~stanza_flags:s.buildable.flags
+        ~modules_flags
         ~sctx
         ~loc
         ~coqc_dir:dir
@@ -1190,12 +1255,15 @@ let coqtop_args_extraction ~sctx ~dir (s : Coq_stanza.Extraction.t) coq_module =
   let boot_type =
     Bootstrap.make ~scope ~use_stdlib ~wrapper_name ~coq_lang_version coq_module
   in
+  let boot_flags = Resolve.Memo.read boot_type |> Action_builder.map ~f:Bootstrap.flags in
+  let per_file_flags = None in
   let* mode = select_native_mode ~sctx ~dir s.buildable in
   generic_coq_args
     ~sctx
     ~dir
     ~wrapper_name
-    ~boot_type
+    ~boot_flags
+    ~per_file_flags
     ~mode
     ~coq_prog:`Coqtop
     ~stanza_flags:s.buildable.flags
