@@ -47,10 +47,10 @@ let missing_run_t (error : Cram_test.t) =
 ;;
 
 let test_rule
-  ~sctx
-  ~dir
-  ({ alias; loc; enabled_if; deps; locks; sandbox; packages = _ } : Spec.t)
-  (test : (Cram_test.t, error) result)
+      ~sctx
+      ~dir
+      ({ alias; loc; enabled_if; deps; locks; sandbox; packages = _ } : Spec.t)
+      (test : (Cram_test.t, error) result)
   =
   let module Alias_rules = Simple_rules.Alias_rules in
   let aliases = Alias.Name.Set.to_list_map alias ~f:(Alias.make ~dir) in
@@ -87,12 +87,11 @@ let test_rule
          and+ locks = locks >>| Path.Set.to_list in
          Action.progn
            [ Cram_exec.action (Path.build script)
-           ; Diff
-               { Diff.optional = true
-               ; mode = Text
-               ; file1 = Path.build script
-               ; file2 = Path.Build.extend_basename script ~suffix:".corrected"
-               }
+           ; Promote.Diff_action.diff
+               ~optional:true
+               ~mode:Text
+               (Path.build script)
+               (Path.Build.extend_basename script ~suffix:".corrected")
            ]
          |> Action.Full.make ~locks ~sandbox
        in
@@ -127,7 +126,6 @@ let collect_stanzas =
 ;;
 
 let rules ~sctx ~dir tests =
-  let open Memo.O in
   let* stanzas = collect_stanzas ~dir
   and* with_package_mask =
     Dune_load.mask ()
