@@ -1,4 +1,5 @@
 open Import
+open Memo.O
 
 (* This module expands either a (library ... (ctypes ...)) rule or an
    (executables ... (ctypes ...)) rule into the generated set of .ml and .c
@@ -54,9 +55,6 @@ open Import
    generation. As a result, there are no intermediate libraries (or
    packages). *)
 
-module Buildable = Dune_file.Buildable
-module Library = Dune_file.Library
-
 let verbatimf fmt = Printf.ksprintf (fun s -> Pp.concat [ Pp.verbatim s; Pp.newline ]) fmt
 
 let write_c_types_includer_module ~type_description_functor ~c_generated_types_module =
@@ -70,10 +68,10 @@ let write_c_types_includer_module ~type_description_functor ~c_generated_types_m
 ;;
 
 let write_entry_point_module
-  ~ctypes
-  ~type_description_instance
-  ~function_description
-  ~c_types_includer_module
+      ~ctypes
+      ~type_description_instance
+      ~function_description
+      ~c_types_includer_module
   =
   let contents =
     Pp.concat
@@ -125,11 +123,11 @@ let type_gen_gen ~expander ~headers ~type_description_functor =
 ;;
 
 let function_gen_gen
-  ~expander
-  ~(concurrency : Ctypes_field.Concurrency_policy.t)
-  ~(errno_policy : Ctypes_field.Errno_policy.t)
-  ~headers
-  ~function_description_functor
+      ~expander
+      ~(concurrency : Ctypes_field.Concurrency_policy.t)
+      ~(errno_policy : Ctypes_field.Errno_policy.t)
+      ~headers
+      ~function_description_functor
   =
   let open Action_builder.O in
   let module_name = Module_name.to_string function_description_functor in
@@ -169,14 +167,14 @@ let function_gen_gen
 ;;
 
 let build_c_program
-  ~foreign_archives_deps
-  ~sctx
-  ~dir
-  ~source_files
-  ~scope
-  ~cflags
-  ~output
-  ~deps
+      ~foreign_archives_deps
+      ~sctx
+      ~dir
+      ~source_files
+      ~scope
+      ~cflags
+      ~output
+      ~deps
   =
   let ctx = Super_context.context sctx in
   let ocaml = Context.ocaml ctx in
@@ -227,7 +225,8 @@ let build_c_program
       Lib.DB.resolve (Scope.libs scope) (Loc.none, ctypes) |> Resolve.Memo.read
     in
     let ctypes_include_dirs =
-      Lib_flags.L.include_paths [ lib ] (Ocaml Native) |> Path.Set.to_list
+      Lib_flags.L.include_paths [ lib ] (Ocaml Native) ocaml.lib_config
+      |> Path.Set.to_list
     in
     let include_dirs = ocaml_where :: ctypes_include_dirs in
     Command.Args.S
@@ -298,7 +297,6 @@ let gen_rules ~cctx ~(buildable : Buildable.t) ~loc ~scope ~dir ~sctx =
   let type_description_functor = ctypes.type_description.functor_ in
   let c_types_includer_module = ctypes.generated_types in
   let c_generated_types_module = Ctypes_field.c_generated_types_module ctypes in
-  let open Memo.O in
   let foreign_archives_deps =
     let { Lib_config.ext_lib; ext_dll; _ } =
       (Compilation_context.ocaml cctx).lib_config
@@ -334,7 +332,6 @@ let gen_rules ~cctx ~(buildable : Buildable.t) ~loc ~scope ~dir ~sctx =
       |> Memo.return
     | Pkg_config ->
       let+ () =
-        let open Memo.O in
         let setup query =
           let* res = Pkg_config.gen_rule sctx ~dir ~loc query in
           match res with
