@@ -4,16 +4,22 @@ open Memo.O
 let first_exe (exes : Executables.t) = snd (Nonempty_list.hd exes.names)
 
 let linkages
-  ~dynamically_linked_foreign_archives
-  (ocaml : Ocaml_toolchain.t)
-  ~(exes : Executables.t)
-  ~explicit_js_mode
+      ~dynamically_linked_foreign_archives
+      (ocaml : Ocaml_toolchain.t)
+      ~(exes : Executables.t)
+      ~explicit_js_mode
+      ~jsoo_enabled_modes
+      ~jsoo_is_whole_program
   =
   let module L = Executables.Link_mode in
   let l =
     let has_native = Result.is_ok ocaml.ocamlopt in
     let modes =
       L.Map.to_list exes.modes
+      |> List.filter ~f:(fun (mode, _) ->
+        match (mode : Executables.Link_mode.t) with
+        | Jsoo mode -> Js_of_ocaml.Mode.Pair.select ~mode jsoo_enabled_modes
+        | Byte_complete | Other _ -> true)
       |> List.map ~f:(fun (mode, loc) ->
         Exe.Linkage.of_user_config ocaml ~dynamically_linked_foreign_archives ~loc mode)
     in
@@ -23,21 +29,39 @@ let linkages
       else List.filter modes ~f:(fun x -> not (Exe.Linkage.is_native x))
     in
     let modes =
-      if L.Map.mem exes.modes L.js
-      then Exe.Linkage.byte_for_jsoo :: modes
+      if L.Map.existsi ~f:(fun m _ -> L.is_jsoo m) exes.modes
+      then (
+        let jsoo_bytecode_exe_needed =
+          Js_of_ocaml.Mode.Set.inter jsoo_enabled_modes jsoo_is_whole_program
+        in
+        let bytecode_exe_needed =
+          L.Map.existsi
+            ~f:(fun mode _ ->
+              match (mode : Executables.Link_mode.t) with
+              | Jsoo mode -> Js_of_ocaml.Mode.Pair.select ~mode jsoo_bytecode_exe_needed
+              | Byte_complete | Other _ -> false)
+            exes.modes
+        in
+        if bytecode_exe_needed then Exe.Linkage.byte_for_jsoo :: modes else modes)
       else if explicit_js_mode
       then modes
       else if L.Map.mem exes.modes L.byte
-      then Exe.Linkage.js :: Exe.Linkage.byte_for_jsoo :: modes
+      then
+        Exe.Linkage.js
+        ::
+        (if Js_of_ocaml.Mode.Pair.select ~mode:JS jsoo_is_whole_program
+         then Exe.Linkage.byte_for_jsoo :: modes
+         else modes)
       else modes
     in
     modes
   in
   (* If bytecode was requested but not native or best version, add custom
      linking *)
-  if L.Map.mem exes.modes L.byte
-     && (not (L.Map.mem exes.modes L.native))
-     && not (L.Map.mem exes.modes L.exe)
+  if
+    L.Map.mem exes.modes L.byte
+    && (not (L.Map.mem exes.modes L.native))
+    && not (L.Map.mem exes.modes L.exe)
   then Exe.Linkage.custom ocaml.version :: l
   else l
 ;;
@@ -66,13 +90,13 @@ let programs ~modules ~(exes : Executables.t) =
 ;;
 
 let o_files
-  sctx
-  ~dir
-  ~expander
-  ~(exes : Executables.t)
-  ~linkages
-  ~dir_contents
-  ~requires_compile
+      sctx
+      ~dir
+      ~expander
+      ~(exes : Executables.t)
+      ~linkages
+      ~dir_contents
+      ~requires_compile
   =
   if not (Executables.has_foreign exes)
   then Memo.return @@ Mode.Map.empty
@@ -113,14 +137,14 @@ let o_files
 ;;
 
 let executables_rules
-  ~sctx
-  ~dir
-  ~expander
-  ~dir_contents
-  ~scope
-  ~compile_info
-  ~embed_in_plugin_libraries
-  (exes : Executables.t)
+      ~sctx
+      ~dir
+      ~expander
+      ~dir_contents
+      ~scope
+      ~compile_info
+      ~embed_in_plugin_libraries
+      (exes : Executables.t)
   =
   (* Use "eobjs" rather than "objs" to avoid a potential conflict with a library
      of the same name *)
@@ -134,11 +158,22 @@ let executables_rules
   let* ocaml = Context.ocaml ctx in
   let project = Scope.project scope in
   let explicit_js_mode = Dune_project.explicit_js_mode project in
+  let js_of_ocaml = Js_of_ocaml.In_context.make ~dir exes.buildable.js_of_ocaml in
   let* linkages =
-    let+ dynamically_linked_foreign_archives =
+    let+ jsoo_enabled_modes =
+      Jsoo_rules.jsoo_enabled_modes ~expander ~dir ~in_context:js_of_ocaml
+    and+ jsoo_is_whole_program =
+      Jsoo_rules.jsoo_is_whole_program sctx ~dir ~in_context:js_of_ocaml
+    and+ dynamically_linked_foreign_archives =
       Context.dynamically_linked_foreign_archives ctx
     in
-    linkages ocaml ~dynamically_linked_foreign_archives ~exes ~explicit_js_mode
+    linkages
+      ocaml
+      ~dynamically_linked_foreign_archives
+      ~exes
+      ~explicit_js_mode
+      ~jsoo_enabled_modes
+      ~jsoo_is_whole_program
   in
   let* flags = Buildable_rules.ocaml_flags sctx ~dir exes.buildable.flags in
   let* modules, pp =
@@ -158,10 +193,12 @@ let executables_rules
     let requires_compile = Lib.Compile.direct_requires compile_info in
     let requires_link = Lib.Compile.requires_link compile_info in
     let js_of_ocaml =
-      let js_of_ocaml = Js_of_ocaml.In_context.make ~dir exes.buildable.js_of_ocaml in
-      if explicit_js_mode
-      then Option.some_if (List.exists linkages ~f:Exe.Linkage.is_js) js_of_ocaml
-      else Some js_of_ocaml
+      Js_of_ocaml.Mode.Pair.mapi
+        ~f:(fun mode x ->
+          Option.some_if
+            ((not explicit_js_mode) || List.exists linkages ~f:(Exe.Linkage.is_jsoo ~mode))
+            x)
+        js_of_ocaml
     in
     Compilation_context.create
       ()
@@ -182,7 +219,7 @@ let executables_rules
   let lib_config = ocaml.lib_config in
   let stdlib_dir = lib_config.stdlib_dir in
   let* requires_compile = Compilation_context.requires_compile cctx in
-  let* requires_link = Compilation_context.requires_link cctx in
+  let* requires_hidden = Compilation_context.requires_hidden cctx in
   let* dep_graphs =
     (* Building an archive for foreign stubs, we link the corresponding object
        files directly to improve perf. *)
@@ -266,17 +303,17 @@ let executables_rules
   in
   ( cctx
   , Merlin.make
-      ~requires:requires_link
+      ~requires_compile
+      ~requires_hidden
       ~stdlib_dir
       ~flags
       ~modules
-      ~source_dirs:Path.Source.Set.empty
       ~libname:None
       ~obj_dir
       ~preprocess:
         (Preprocess.Per_module.without_instrumentation exes.buildable.preprocess)
       ~dialects:(Dune_project.dialects (Scope.project scope))
-      ~ident:(Lib.Compile.merlin_ident compile_info)
+      ~ident:(Merlin_ident.for_exes ~names:(Nonempty_list.map ~f:snd exes.names))
       ~modes:`Exe )
 ;;
 
@@ -290,17 +327,14 @@ let compile_info ~scope (exes : Executables.t) =
          ~instrumentation_backend:(Lib.DB.instrumentation_backend (Scope.libs scope)))
     >>| Preprocess.Per_module.pps
   in
-  let names = Nonempty_list.to_list exes.names in
-  let merlin_ident = Merlin_ident.for_exes ~names:(List.map ~f:snd names) in
   Lib.DB.resolve_user_written_deps
     (Scope.libs scope)
-    (`Exe names)
+    (`Exe exes.names)
     exes.buildable.libraries
     ~pps
     ~dune_version
     ~allow_overlaps:exes.buildable.allow_overlapping_dependencies
     ~forbidden_libraries:exes.forbidden_libraries
-    ~merlin_ident
 ;;
 
 let rules ~sctx ~dir ~dir_contents ~scope ~expander (exes : Executables.t) =
@@ -321,5 +355,6 @@ let rules ~sctx ~dir ~dir_contents ~scope ~expander (exes : Executables.t) =
     let requires_link = Lib.Compile.requires_link compile_info in
     Bootstrap_info.gen_rules sctx exes ~dir ~requires_link
   in
-  Buildable_rules.with_lib_deps (Super_context.context sctx) compile_info ~dir ~f
+  let merlin_ident = Merlin_ident.for_exes ~names:(Nonempty_list.map ~f:snd exes.names) in
+  Buildable_rules.with_lib_deps (Super_context.context sctx) merlin_ident ~dir ~f
 ;;
