@@ -99,6 +99,9 @@ include struct
   module Section = Section
   module Package_dependency = Package_dependency
   module Package_constraint = Package_constraint
+  module Dune_project_name = Dune_project_name
+  module Package = Package
+  module Dialect = Dialect
 end
 
 include Dune_engine.No_io
@@ -110,9 +113,9 @@ module Build_config = struct
     module Rules = Rules
 
     let make
-      ?(build_dir_only_sub_dirs = Rules.empty.build_dir_only_sub_dirs)
-      ?(directory_targets = Rules.empty.directory_targets)
-      rules
+          ?(build_dir_only_sub_dirs = Rules.empty.build_dir_only_sub_dirs)
+          ?(directory_targets = Rules.empty.directory_targets)
+          rules
       =
       let rules = { Rules.build_dir_only_sub_dirs; directory_targets; rules } in
       Gen_rules_result.rules_here rules
