@@ -52,8 +52,8 @@ module Backend = struct
          ; field_i
              "list_partitions_flags"
              (function
-              | None -> []
-              | Some x -> Ordered_set_lang.Unexpanded.encode x)
+               | None -> []
+               | Some x -> Ordered_set_lang.Unexpanded.encode x)
              t.list_partitions_flags
          ; field_o
              "generate_runner"
@@ -68,7 +68,7 @@ module Mode_conf = struct
   module T = struct
     type t =
       | Byte
-      | Javascript
+      | Jsoo of Js_of_ocaml.Mode.t
       | Native
       | Best
 
@@ -77,9 +77,9 @@ module Mode_conf = struct
       | Byte, Byte -> Eq
       | Byte, _ -> Lt
       | _, Byte -> Gt
-      | Javascript, Javascript -> Eq
-      | Javascript, _ -> Lt
-      | _, Javascript -> Gt
+      | Jsoo m, Jsoo m' -> Js_of_ocaml.Mode.compare m m'
+      | Jsoo _, _ -> Lt
+      | _, Jsoo _ -> Gt
       | Native, Native -> Eq
       | Native, _ -> Lt
       | _, Native -> Gt
@@ -94,12 +94,16 @@ module Mode_conf = struct
 
   let to_string = function
     | Byte -> "byte"
-    | Javascript -> "js"
+    | Jsoo JS -> "js"
+    | Jsoo Wasm -> "wasm"
     | Native -> "native"
     | Best -> "best"
   ;;
 
-  let decode = enum [ "byte", Byte; "js", Javascript; "native", Native; "best", Best ]
+  let decode =
+    enum [ "byte", Byte; "js", Jsoo JS; "native", Native; "best", Best ]
+    <|> sum [ "wasm", Syntax.since Stanza.syntax (3, 17) >>> return (Jsoo Wasm) ]
+  ;;
 
   module O = Comparable.Make (T)
   module Map = O.Map
@@ -179,3 +183,9 @@ module Tests = struct
      purposes *)
   let encode _t = assert false
 end
+
+let inline_test_dirname lib_name =
+  sprintf ".%s.inline-tests" (Lib_name.Local.to_string lib_name)
+;;
+
+let inline_test_runner = "inline-test-runner"
