@@ -1,20 +1,18 @@
-open! Dune_engine
-open! Stdune
-open Dune_file
+open Import
 
-val foreign_rules :
-     Foreign.Library.t
+val foreign_rules
+  :  Foreign_library.t
   -> sctx:Super_context.t
   -> expander:Expander.t
   -> dir:Path.Build.t
   -> dir_contents:Dir_contents.t
-  -> unit Memo.Build.t
+  -> unit Memo.t
 
-val rules :
-     Library.t
+val rules
+  :  Library.t
   -> sctx:Super_context.t
   -> dir_contents:Dir_contents.t
   -> dir:Path.Build.t
   -> expander:Expander.t
   -> scope:Scope.t
-  -> (Compilation_context.t * Merlin.t) Memo.Build.t
+  -> (Compilation_context.t * Merlin.t) Memo.t
