@@ -345,14 +345,14 @@ let virtual_modules ~lookup_vlib ~libs vlib =
 ;;
 
 let make_lib_modules
-  ~expander
-  ~dir
-  ~libs
-  ~lookup_vlib
-  ~(lib : Library.t)
-  ~modules
-  ~include_subdirs:(loc_include_subdirs, (include_subdirs : Include_subdirs.t))
-  ~version
+      ~expander
+      ~dir
+      ~libs
+      ~lookup_vlib
+      ~(lib : Library.t)
+      ~modules
+      ~include_subdirs:(loc_include_subdirs, (include_subdirs : Include_subdirs.t))
+      ~version
   =
   let open Resolve.Memo.O in
   let* kind, main_module_name, wrapped =
@@ -498,7 +498,7 @@ let modules_of_stanzas =
       let enabled_if =
         match Stanza.repr stanza with
         | Library.T lib -> lib.enabled_if
-        | Tests.T exes -> exes.build_if
+        | Tests.T tests -> tests.exes.enabled_if
         | Executables.T exes -> exes.enabled_if
         | Melange_stanzas.Emit.T mel -> mel.enabled_if
         | _ -> Blang.true_
@@ -533,12 +533,13 @@ let modules_of_stanzas =
          | Melange_stanzas.Emit.T mel ->
            let obj_dir = Obj_dir.make_melange_emit ~dir ~name:mel.target in
            let+ sources, modules =
+             let version = Dune_project.dune_version project in
              Modules_field_evaluator.eval
                ~expander
                ~modules
                ~stanza_loc:mel.loc
                ~kind:Modules_field_evaluator.Exe_or_normal_lib
-               ~version:mel.dune_version
+               ~version
                ~private_modules:Ordered_set_lang.Unexpanded.standard
                ~src_dir:dir
                mel.modules
@@ -555,16 +556,16 @@ let modules_of_stanzas =
 ;;
 
 let make
-  dune_file
-  ~expander
-  ~dir
-  ~libs
-  ~project
-  ~lib_config
-  ~loc
-  ~lookup_vlib
-  ~include_subdirs:(loc_include_subdirs, (include_subdirs : Include_subdirs.t))
-  ~dirs
+      dune_file
+      ~expander
+      ~dir
+      ~libs
+      ~project
+      ~lib_config
+      ~loc
+      ~lookup_vlib
+      ~include_subdirs:(loc_include_subdirs, (include_subdirs : Include_subdirs.t))
+      ~dirs
   =
   let+ modules_of_stanzas =
     let modules =
