@@ -1,4 +1,5 @@
 open Import
+open Memo.O
 
 module Common = struct
   module Encode = struct
@@ -72,7 +73,6 @@ module Stdlib = struct
   let map t ~f = { t with modules = Module_name.Map.map t.modules ~f }
 
   let traverse t ~f =
-    let open Memo.O in
     let+ modules = Module_name.Parallel_map.parallel_map t.modules ~f:(fun _ -> f) in
     { t with modules }
   ;;
@@ -421,14 +421,12 @@ module Group = struct
 
   module Memo_traversals = struct
     let rec parallel_map ({ alias; modules; name = _ } as t) ~f =
-      let open Memo.O in
       let+ alias, modules =
         Memo.fork_and_join (fun () -> f alias) (fun () -> parallel_map_modules modules ~f)
       in
       { t with alias; modules }
 
     and parallel_map_modules modules ~f =
-      let open Memo.O in
       Module_name.Parallel_map.parallel_map modules ~f:(fun _ n ->
         match n with
         | Module m ->
@@ -440,8 +438,6 @@ module Group = struct
     ;;
   end
 
-  let group_interfaces (t : t) m = parents t m |> List.map ~f:lib_interface
-
   let make_alias_for t m ~parents =
     match Module.kind m with
     | Alias _ | Wrapped_compat -> []
@@ -660,26 +656,7 @@ module Wrapped = struct
       ]
   ;;
 
-  (* TODO remove this eventually *)
-  let old_decode ~src_dir =
-    let open Dune_lang.Decoder in
-    let open Common.Decode in
-    fields
-      (let+ main_module_name = main_module_name
-       and+ modules = modules ~src_dir ()
-       and+ wrapped_compat = modules ~name:"wrapped_compat" ~src_dir ()
-       and+ alias = field "alias_module" (Module.decode ~src_dir)
-       and+ wrapped = field "wrapped" Dune_lang.Wrapped.decode in
-       let group =
-         { Group.alias
-         ; name = main_module_name
-         ; modules = modules |> Module_name.Map.map ~f:(fun m -> Group.Module m)
-         }
-       in
-       { group; wrapped_compat; wrapped; toplevel_module = `Exported })
-  ;;
-
-  let new_decode ~src_dir =
+  let decode ~src_dir =
     let open Dune_lang.Decoder in
     let open Common.Decode in
     fields
@@ -689,11 +666,6 @@ module Wrapped = struct
        { group; wrapped_compat; wrapped; toplevel_module = `Exported })
   ;;
 
-  let decode ~src_dir =
-    let open Dune_lang.Decoder in
-    new_decode ~src_dir <|> old_decode ~src_dir
-  ;;
-
   let map ({ group; wrapped_compat; toplevel_module = _; wrapped = _ } as t) ~f =
     { t with
       group = Group.map group ~f
@@ -745,7 +717,6 @@ module Wrapped = struct
 
   let find t name = Group.find t.group name
   let find_dep t ~of_ name = Group.find_dep t.group ~of_ name
-  let group_interfaces (t : t) m = Group.group_interfaces t.group m
   let alias_for t m = Group.alias_for t.group m
 end
 
@@ -909,7 +880,6 @@ let fold_user_available t ~f ~init =
 
 let map_user_written t ~f =
   let f m = if is_user_written m then f m else Memo.return m in
-  let open Memo.O in
   let+ modules =
     match t.modules with
     | Singleton m ->
@@ -1304,19 +1274,6 @@ module With_vlib = struct
          | Impl { impl; vlib = _; _ } -> alias_for impl m)
   ;;
 
-  let group_interfaces =
-    let group_interfaces t m =
-      match t.modules with
-      | Wrapped w -> Wrapped.group_interfaces w m
-      | Singleton w -> [ w ]
-      | _ -> []
-    in
-    fun t m ->
-      match t with
-      | Modules t -> group_interfaces t m
-      | Impl { impl; vlib; _ } -> group_interfaces impl m @ group_interfaces vlib m
-  ;;
-
   let local_open t m =
     alias_for t m
     |> List.map ~f:(fun m ->
