@@ -9,3 +9,8 @@ val library_path : string list
 
 (** Where to install files. All the directories are absolute path *)
 val roots : string option Install.Roots.t
+
+val toolchains : Dune_config.Config.Toggle.t
+val pkg_build_progress : Dune_config.Config.Toggle.t
+val lock_dev_tool : Dune_config.Config.Toggle.t
+val prefix : string option
