@@ -16,14 +16,14 @@ module Lock_dir = struct
     }
 
   let to_dyn
-    { path
-    ; version_preference
-    ; solver_env
-    ; unset_solver_vars
-    ; repositories
-    ; constraints
-    ; pins
-    }
+        { path
+        ; version_preference
+        ; solver_env
+        ; unset_solver_vars
+        ; repositories
+        ; constraints
+        ; pins
+        }
     =
     Dyn.record
       [ "path", Path.Source.to_dyn path
@@ -41,14 +41,14 @@ module Lock_dir = struct
   ;;
 
   let hash
-    { path
-    ; version_preference
-    ; solver_env
-    ; unset_solver_vars
-    ; repositories
-    ; constraints
-    ; pins
-    }
+        { path
+        ; version_preference
+        ; solver_env
+        ; unset_solver_vars
+        ; repositories
+        ; constraints
+        ; pins
+        }
     =
     Poly.hash
       ( path
@@ -61,15 +61,15 @@ module Lock_dir = struct
   ;;
 
   let equal
-    { path
-    ; version_preference
-    ; solver_env
-    ; unset_solver_vars
-    ; repositories
-    ; constraints
-    ; pins
-    }
-    t
+        { path
+        ; version_preference
+        ; solver_env
+        ; unset_solver_vars
+        ; repositories
+        ; constraints
+        ; pins
+        }
+        t
     =
     Path.Source.equal path t.path
     && Option.equal
@@ -313,20 +313,20 @@ module Context = struct
     ;;
 
     let equal
-      { loc = _
-      ; profile
-      ; targets
-      ; env
-      ; toolchain
-      ; name
-      ; host_context
-      ; paths
-      ; fdo_target_exe
-      ; dynamically_linked_foreign_archives
-      ; instrument_with
-      ; merlin
-      }
-      t
+          { loc = _
+          ; profile
+          ; targets
+          ; env
+          ; toolchain
+          ; name
+          ; host_context
+          ; paths
+          ; fdo_target_exe
+          ; dynamically_linked_foreign_archives
+          ; instrument_with
+          ; merlin
+          }
+          t
       =
       Profile.equal profile t.profile
       && List.equal Target.equal targets t.targets
@@ -462,6 +462,14 @@ module Context = struct
       Common.equal base t.base && Opam_switch.equal switch t.switch
     ;;
 
+    let name_hint_opt name =
+      if String.is_prefix ~prefix:"/" name
+      then (
+        let context_name = Filename.basename name in
+        Some [ Pp.textf "(name %s) would be a valid context name" context_name ])
+      else None
+    ;;
+
     let decode =
       let+ loc_switch, switch = field "switch" (located string)
       and+ name = field_o "name" Context_name.decode
@@ -479,9 +487,10 @@ module Context = struct
              | None ->
                User_error.raise
                  ~loc:loc_switch
-                 [ Pp.textf "Generated context name %S is invalid" name
-                 ; Pp.text
-                     "Please specify a context name manually with the (name ..) field"
+                 ?hints:(name_hint_opt name)
+                 [ Pp.textf
+                     "The name generated from this switch can not be used as a context \
+                      name. Please use (name) to disambiguate."
                  ])
         in
         let base = { base with targets = Target.add base.targets x; name } in
@@ -629,10 +638,10 @@ type t =
   ; repos : Dune_pkg.Pkg_workspace.Repository.t list
   ; lock_dirs : Lock_dir.t list
   ; dir : Path.Source.t
-  ; sources : Dune_pkg.Pin_stanza.DB.Workspace.t
+  ; pins : Dune_pkg.Pin_stanza.DB.Workspace.t
   }
 
-let to_dyn { merlin_context; contexts; env; config; repos; lock_dirs; sources; dir } =
+let to_dyn { merlin_context; contexts; env; config; repos; lock_dirs; pins; dir } =
   let open Dyn in
   record
     [ "merlin_context", option Context_name.to_dyn merlin_context
@@ -642,11 +651,11 @@ let to_dyn { merlin_context; contexts; env; config; repos; lock_dirs; sources; d
     ; "repos", list Repository.to_dyn repos
     ; "solver", (list Lock_dir.to_dyn) lock_dirs
     ; "dir", Path.Source.to_dyn dir
-    ; "sources", Dune_pkg.Pin_stanza.DB.Workspace.to_dyn sources
+    ; "pins", Dune_pkg.Pin_stanza.DB.Workspace.to_dyn pins
     ]
 ;;
 
-let equal { merlin_context; contexts; env; config; repos; lock_dirs; dir; sources } w =
+let equal { merlin_context; contexts; env; config; repos; lock_dirs; dir; pins } w =
   Option.equal Context_name.equal merlin_context w.merlin_context
   && List.equal Context.equal contexts w.contexts
   && Option.equal Dune_env.equal env w.env
@@ -654,10 +663,10 @@ let equal { merlin_context; contexts; env; config; repos; lock_dirs; dir; source
   && List.equal Repository.equal repos w.repos
   && List.equal Lock_dir.equal lock_dirs w.lock_dirs
   && Path.Source.equal dir w.dir
-  && Dune_pkg.Pin_stanza.DB.Workspace.equal sources w.sources
+  && Dune_pkg.Pin_stanza.DB.Workspace.equal pins w.pins
 ;;
 
-let hash { merlin_context; contexts; env; config; repos; lock_dirs; dir; sources } =
+let hash { merlin_context; contexts; env; config; repos; lock_dirs; dir; pins } =
   Poly.hash
     ( Option.hash Context_name.hash merlin_context
     , List.hash Context.hash contexts
@@ -666,13 +675,15 @@ let hash { merlin_context; contexts; env; config; repos; lock_dirs; dir; sources
     , List.hash Repository.hash repos
     , List.hash Lock_dir.hash lock_dirs
     , Path.Source.hash dir
-    , Dune_pkg.Pin_stanza.DB.Workspace.hash sources )
+    , Dune_pkg.Pin_stanza.DB.Workspace.hash pins )
 ;;
 
 let find_lock_dir t path =
   List.find t.lock_dirs ~f:(fun lock_dir -> Path.Source.equal lock_dir.path path)
 ;;
 
+let add_repo t repo = { t with repos = repo :: t.repos }
+
 include Dune_lang.Versioned_file.Make (struct
     type t = unit
   end)
@@ -690,13 +701,13 @@ module Clflags = struct
     }
 
   let to_dyn
-    { x
-    ; profile
-    ; instrument_with
-    ; workspace_file
-    ; config_from_command_line
-    ; config_from_config_file
-    }
+        { x
+        ; profile
+        ; instrument_with
+        ; workspace_file
+        ; config_from_command_line
+        ; config_from_config_file
+        }
     =
     let open Dyn in
     record
@@ -762,9 +773,9 @@ let top_sort contexts =
 ;;
 
 let create_final_config
-  ~config_from_config_file
-  ~config_from_command_line
-  ~config_from_workspace_file
+      ~config_from_config_file
+      ~config_from_command_line
+      ~config_from_workspace_file
   =
   let ( ++ ) = Dune_config.superpose in
   Dune_config.default
@@ -834,7 +845,7 @@ let step1 clflags =
          ~default:(lazy []))
   and+ config_from_workspace_file = Dune_config.decode_fields_of_workspace_file
   and+ lock_dirs = multi_field "lock_dir" (Lock_dir.decode ~dir)
-  and+ sources = Dune_pkg.Pin_stanza.DB.Workspace.decode in
+  and+ pins = Dune_pkg.Pin_stanza.DB.Workspace.decode in
   let+ contexts = multi_field "context" (lazy_ Context.decode) in
   let config =
     create_final_config
@@ -894,9 +905,10 @@ let step1 clflags =
          match merlin_context with
          | Some _ -> merlin_context
          | None ->
-           if List.exists contexts ~f:(function
-                | Context.Default _ -> true
-                | _ -> false)
+           if
+             List.exists contexts ~f:(function
+               | Context.Default _ -> true
+               | _ -> false)
            then Some Context_name.default
            else None
        in
@@ -907,7 +919,7 @@ let step1 clflags =
        ; repos
        ; lock_dirs
        ; dir
-       ; sources
+       ; pins
        })
   in
   { Step1.t; config }
@@ -940,7 +952,7 @@ let default clflags =
   ; repos = default_repositories
   ; lock_dirs = []
   ; dir = Path.Source.root
-  ; sources = Dune_pkg.Pin_stanza.DB.Workspace.empty
+  ; pins = Dune_pkg.Pin_stanza.DB.Workspace.empty
   }
 ;;
 
